#pragma required_pointer_size long
#ifndef lint
static const char sccsid[] = "@(#)basic_type.c	1.1     2/10/94";
#endif
/*
*****************************************************************************
**                                                                          *
**                         COPYRIGHT (c) 1991 BY                            *
**             DIGITAL EQUIPMENT CORPORATION, MAYNARD, MASS.                *
**                         ALL RIGHTS RESERVED                              *
**                                                                          *
**  THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND  COPIED  *
**  ONLY  IN  ACCORDANCE  WITH  THE  TERMS  OF  SUCH  LICENSE AND WITH THE  *
**  INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR  ANY  OTHER  *
**  COPIES  THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY  *
**  OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF  THE  SOFTWARE  IS  HEREBY  *
**  TRANSFERRED.                                                            *
**                                                                          *
**  THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE  WITHOUT  NOTICE  *
**  AND  SHOULD  NOT  BE  CONSTRUED  AS  A COMMITMENT BY DIGITAL EQUIPMENT  *
**  CORPORATION.                                                            *
**                                                                          *
**  DIGITAL ASSUMES NO RESPONSIBILITY FOR THE USE OR  RELIABILITY  OF  ITS  *
**  SOFTWARE ON EQUIPMENT WHICH IS NOT SUPPLIED BY DIGITAL.                 *
**                                                                          *
*****************************************************************************
**++
**  PRODUCT:
***
**	DEC C++ Demangler
**
**  FILE:  
**
**      basic_type.c
**
**  ABSTRACT:
**
**	Grammar element that parses a basic_type.
**
**  ENVIRONMENT:
**
**	User Mode
**
**  AUTHORS:
**
** 	Shell S. Simpson (DECwest Engineering)
**
**  MODIFICATION HISTORY:
**
**  21-Oct-1997	Shi-Jung Kao
**	Parse the expanded basic types.
**
**  20-Jan-2000 Shi-Jung Kao
**	Under version7, Exclude "LM0" from basic type of long long.
**	Added DEMANGLE_TRACEIN/TRACEOUT.
**
**--
**/


#ifdef CXX_NT
#include "defs.h"
#include "mn_nm.h"
#include "gram_elm.h"
#include "util.h"
#else
#include "demangle.h"
#include "grammar_elements.h"
#endif

#include "demangle_internal.h"

#include <stdlib.h>
#include <stddef.h>
#include <stdio.h>
#include <string.h>

/* language element particular definitions */
#define ASCII_ZERO 48

int /* recognition status */
_basic_type(const char* s,	/* IN:  string to be parsed */
           int start,		/* IN:  index of first unparsed character */
       	   int* end,		/* OUT: index of last parsed character */
           int* bt)		/* OUT: integer containing basic type number */
{
  /* variables common to all language elements */
  int ci;  /* current index */
  char ch; /* current character */
  
  /* variables particular to this language element */
  char basic_type_tokens[NUMBER_OF_BASIC_TYPES + 1]   = BASIC_TYPE_TOKENS;
  char* basic3_type_tokens[NUMBER_OF_BASIC3_TYPES + 1]   = BASIC3_TYPE_TOKENS;
  char* basic_type_strings[NUMBER_OF_BASIC_TYPES +
			  NUMBER_OF_BASIC3_TYPES + 1] = BASIC_TYPE_STRINGS;

  int i; /* temporary index variable */
  char* temp;

  DEMANGLE_TRACEIN(_basic_type);
  if (s[start] == 'L' && strlen(s) > start+1 && s[start+1] == 'M') {
    *end = start - 1; /* Its a pointer to member type. */
    DEMANGLE_TRACEOUT(_basic_type);
    return MLD_NOTHING_RECOGNIZED;
  }

  /*
  ***
  *** Check to see if the current character is one of the basic type 
  *** characters.  If so, return the corresponding number 
  */

  ch = s[ci = start]; /* initialize ch and ci */
  for (i = 0 ; i < EXPANDED_BASIC1_TYPE - 1; i++)
  {
    if (ch == basic_type_tokens[i])
    {
      *end = ci; /* we just recognized the currect character */
      /* copy basic type string over into bts */
      *bt = i; /* index of the basic type */
      DEMANGLE_TRACEOUT(_basic_type);
      return MLD_SOMETHING_RECOGNIZED;
    }
  }
  if (ch == basic_type_tokens[EXPANDED_BASIC1_TYPE-1]) {
    ch = s[++ci];
    for (i = EXPANDED_BASIC1_TYPE ; i < EXPANDED_BASIC2_TYPE - 1; i++) {
      if (ch == basic_type_tokens[i]) {
	*end = ci; /* we just recognized the currect character */
	/* copy basic type string over into bts */
	*bt = i; /* index of the basic type */
	DEMANGLE_TRACEOUT(_basic_type);
	return MLD_SOMETHING_RECOGNIZED;
      }
    }
  }
  if (ch == basic_type_tokens[EXPANDED_BASIC2_TYPE-1]) {
    ch = s[++ci];
    for (i = EXPANDED_BASIC2_TYPE ; i < NUMBER_OF_BASIC_TYPES ; i++) {
      if (ch == basic_type_tokens[i]) {
	*end = ci; /* we just recognized the currect character */
	/* copy basic type string over into bts */
	*bt = i; /* index of the basic type */
	DEMANGLE_TRACEOUT(_basic_type);
	return MLD_SOMETHING_RECOGNIZED;
      }
    }
  }

  for (i = 0; i < NUMBER_OF_BASIC3_TYPES; i++) {
    if (strncmp(&s[ci = start], basic3_type_tokens[i], EXPANDED_BASIC3_SIZE) == 0) {
      *end = ci + 2;
      /* copy basic type string over into bts */
      *bt = i+NUMBER_OF_BASIC_TYPES; /* index of the basic type */
      DEMANGLE_TRACEOUT(_basic_type);
      return MLD_SOMETHING_RECOGNIZED;
    }
  }

  if (ch == NONTYPE_TEMPLATE_TOKEN) {
    /* template argument has form of ZnZ or Zn_mZ */
    ch = s[++ci];
    if (ch <= '9' && ch > '0') {
      for (ch = s[++ci]; ch <= '9' && ch >= '0';)
	ch = s[++ci];
      if (ch == NONTYPE_TEMPLATE_TOKEN) {
	/* ZnZ format */
	*end = ci;  /* point at Z */
	*bt = NUMBER_OF_BASIC_TYPES;
	DEMANGLE_TRACEOUT(_basic_type);
	return MLD_SOMETHING_RECOGNIZED;
      } else if (ch == '_') {
	/* Zn_mZ format */
	ch = s[++ci];
	if (ch <= '9' && ch > '0') {
	  for (ch = s[++ci]; ch <= '9' && ch >= '0';)
	    ch = s[++ci];
	  if (ch == NONTYPE_TEMPLATE_TOKEN) {
	    *end = ci;  /* point at Z */
	    *bt = NUMBER_OF_BASIC_TYPES;
	    DEMANGLE_TRACEOUT(_basic_type);
	    return MLD_SOMETHING_RECOGNIZED;
	  }
	}  /* if is valid m */
      }  /* ch == '_' */
    }  /* if is valid n */
  }  /* ch == NONTYPE_TEMPLATE_TOKEN */
  /*
  ***
  *** At this point, we know that none of the basic types has been recognized.
  *** 
  */

  *end = start - 1; /* not even the first character!! */
  DEMANGLE_TRACEOUT(_basic_type);
  return MLD_NOTHING_RECOGNIZED;
}
