#ifndef _DEMANGLE_INTERNAL_H_
#define _DEMANGLE_INTERNAL_H_
/*
*****************************************************************************
**                                                                          *
**                         COPYRIGHT (c) 1991 BY                            *
**             DIGITAL EQUIPMENT CORPORATION, MAYNARD, MASS.                *
**                          ALL RIGHTS RESERVED                             *
**                                                                          *
**  THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND  COPIED  *
**  ONLY  IN  ACCORDANCE  WITH  THE  TERMS  OF  SUCH  LICENSE AND WITH THE  *
**  INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR  ANY  OTHER  *
**  COPIES  THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY  *
**  OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF  THE  SOFTWARE  IS  HEREBY  *
**  TRANSFERRED.                                                            *
**                                                                          *
**  THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE  WITHOUT  NOTICE  *
**  AND  SHOULD  NOT  BE  CONSTRUED  AS  A COMMITMENT BY DIGITAL EQUIPMENT  *
**  CORPORATION.                                                            *
**                                                                          *
**  DIGITAL ASSUMES NO RESPONSIBILITY FOR THE USE OR  RELIABILITY  OF  ITS  *
**  SOFTWARE ON EQUIPMENT WHICH IS NOT SUPPLIED BY DIGITAL.                 *
**                                                                          *
*****************************************************************************
**++
**  PRODUCT:
***
**	DEC C++ Demangler
**
**  FILE:  
**
**      demangle_debug.h
**
**  ABSTRACT:
**
**	Header file containing definitions for debugging the demangler.
**
**  ENVIRONMENT:
**
**	User Mode
**
**  AUTHORS:
**
** 	Kenneth Block
**
**  MODIFICATION HISTORY:
**
**  11-Apr-2000	Kenneth Block
**	Pull debugging stuff out of other headers
**
**--
**/

extern int demangle_no_template_details;

#if PRODUCTION
#define DEMANGLE_INIT
#define DEMANGLE_ASSERT(VAR1, VAR2)
#define DEMANGLE_RECORD_ALLOC(VAR)
#define DEMANGLE_TRACEIN(VAR)
#define DEMANGLE_TRACEOUT(VAR)
#define DEMANGLE_TRACE_INIT
#define DEMANGLE_TRACE_FINI
#define DEMANGLE_FREE(VAR)				\
	free(VAR);
#define DEMANGLE_DUMP_ARRAY
#else
void demangle_check_free(void* foo);
void demangle_dump_malloc(void* foo);

extern void* malloc_array[65536];

#define DEMANGLE_TRACE_VALID_VALUE 34567
extern int demangle_trace_valid;
extern int demangle_trace_enable;

extern int demangle_memcheck;
extern int demangle_dump;
extern int my_index;
extern int my_pin_point_malloc;

#define DEMANGLE_INIT					\
	my_index = 0;					\
        my_pin_point_malloc = 0;

#define DEMANGLE_ASSERT(VAR1, VAR2)			\
        if (!VAR1)					\
	  printf (" ==> Demangle Assertion:" #VAR2 " in %s at line: %d\n",	\
		  __FILE__, __LINE__)

#define DEMANGLE_TRACEIN(VAR)				\
        if (demangle_trace_enable)			\
	  printf (" ==> %s:" #VAR "\n",__FILE__)

#define DEMANGLE_TRACEOUT(VAR)				\
        if (demangle_trace_enable)			\
	  printf (" <== " #VAR " at line: %d\n",__LINE__)

#define DEMANGLE_TRACE_INIT				\
	demangle_trace_valid = DEMANGLE_TRACE_VALID_VALUE;

#define DEMANGLE_TRACE_FINI				\
	demangle_trace_valid = 0;

#define DEMANGLE_RECORD_ALLOC(VAR)			\
	if (demangle_memcheck && my_index < 65536 &&    \
	    demangle_trace_valid == DEMANGLE_TRACE_VALID_VALUE) {	\
	  if (demangle_trace_enable)			\
	    printf (" *** malloc %x ==> %s at line: %d\n", VAR,		\
		    __FILE__, __LINE__);		\
	  malloc_array[my_index++] = VAR;		\
          demangle_dump_malloc(VAR);			\
	}

#define DEMANGLE_FREE(VAR)				\
        if (demangle_memcheck && demangle_trace_valid == DEMANGLE_TRACE_VALID_VALUE) {	\
	  if (demangle_trace_enable)			\
	    printf (" *** free %x ==> %s at line: %d\n", VAR, __FILE__, __LINE__);	\
	  demangle_check_free(VAR);			\
        }						\
	free(VAR);

/* Warn once, but then reset array otherwise third degree
   will not find it as a leak. */
#define DEMANGLE_DUMP_ARRAY				\
        if (demangle_memcheck && demangle_trace_valid == DEMANGLE_TRACE_VALID_VALUE) {	\
          int my_i;                                     \
	  for(my_i = 0; my_i < my_index; my_i++)	\
	    if (malloc_array[my_i] != NULL) {		\
	      printf("ERROR demangle===> Left unfree malloc address[%d]: %p\n",my_i, malloc_array[my_i]); \
	      malloc_array[my_i] = NULL;                \
	    }                                           \
          my_index = 0;                                 \
	}           

#endif /* DEMANGLE_PRODUCTION */

#endif /* _DEMANGLE_INTERNAL_H_ */
