#pragma required_pointer_size long
#ifndef lint
static const char sccsid[] = "@(#)function_arguments.c	1.1     2/10/94";
#endif
/*
*****************************************************************************
**                                                                          *
**                         COPYRIGHT (c) 1991 BY                            *
**             DIGITAL EQUIPMENT CORPORATION, MAYNARD, MASS.                *
**                          ALL RIGHTS RESERVED                             *
**                                                                          *
**  THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND  COPIED  *
**  ONLY  IN  ACCORDANCE  WITH  THE  TERMS  OF  SUCH  LICENSE AND WITH THE  *
**  INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR  ANY  OTHER  *
**  COPIES  THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY  *
**  OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF  THE  SOFTWARE  IS  HEREBY  *
**  TRANSFERRED.                                                            *
**                                                                          *
**  THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE  WITHOUT  NOTICE  *
**  AND  SHOULD  NOT  BE  CONSTRUED  AS  A COMMITMENT BY DIGITAL EQUIPMENT  *
**  CORPORATION.                                                            *
**                                                                          *
**  DIGITAL ASSUMES NO RESPONSIBILITY FOR THE USE OR  RELIABILITY  OF  ITS  *
**  SOFTWARE ON EQUIPMENT WHICH IS NOT SUPPLIED BY DIGITAL.                 *
**                                                                          *
*****************************************************************************
**++
**  PRODUCT:
***
**	DEC C++ Demangler
**
**  FILE:  
**
**      function_arguments.c
**
**  ABSTRACT:
**
**	Grammar element that parses a list a function arguments.
**
**  ENVIRONMENT:
**
**	User Mode
**
**  AUTHORS:
**
** 	Shell S. Simpson (DECwest Engineering)
**
**  MODIFICATION HISTORY:
**
** Mar-22-1998	Shi-Jung Kao	cxxc_bugs:5422
**	Collapse "ve" to "e" for foo(...).
**
** Mar-1-2000   Shi-Jung Kao
**	Added DEMANGLE_TRACEIN/TRACEOUT.
**
**--
**/


#ifdef CXX_NT
#include "defs.h"
#include "mn_nm.h"
#include "gram_elm.h"
#include "util.h"
#else
#include "demangle.h"
#include "grammar_elements.h"
#endif

#include "demangle_internal.h"

#include <stdlib.h>
#include <stddef.h>
#include <stdio.h>
#include <string.h>

/* language element particular definitions */
#define ASCII_ZERO 48

int /* recognition status */
_function_arguments (const char* s,		/* IN:  string to be parsed */
                   int start,			/* IN:  index of first unparsed character */
                   int* end,			/* OUT: index of last parsed character */
                   struct MLD_type** fa)	/* OUT: function args consist of type list */
{
  /* variables common to all language elements */
  int ci;  /* current index */
  char ch; /* current character */
  
  /* variables particular to this language element */
  int status;
  int end_of_grammar_element;
  struct MLD_type* a_type = 0L;
  struct MLD_type* temp_fa;
  struct MLD_type* end_of_fa_list;
#if DEC_ANSI_FORM
  struct MLD_type* begin_type;
#endif
  char string[STRSIZ];
  char string2[STRSIZ];
  int end_of_pascal_string;
  int ansi_form = (strstr(s,"__tm__") != NULL);

  DEMANGLE_TRACEIN(_function_arguments);  

  /* Collapse the ve to e */
  if (s[start] == 'v' && strlen(s) > start && s[start+1] == 'e')
    start++;

  /* get first type */
  /* Limit the string to look at */
  status = _pascal_string(s, start, &end_of_pascal_string, string, STRSIZ);
  if (ansi_form && status == MLD_SOMETHING_RECOGNIZED) {
    /* Reconstruct the string, complex type need the begining string. */
    strncpy(&string2[0], s, end_of_pascal_string+1);
    string2[end_of_pascal_string+1] = 0;
    status = _type(&string2[0], start, &end_of_grammar_element, &a_type);
  } else
    status = _type(s, start, &end_of_grammar_element, &a_type); /* allocs mem */

#if DEC_ANSI_FORM
  begin_type = a_type;
#endif

  *fa = 0L; /* initialize fa to point to nothing */
  while (status == MLD_SOMETHING_RECOGNIZED)
  {
     /* if something was recognized, then a_type points to an initialized type
        structure */

     /* append the type modifier to the type modifier list */
     if (*fa == 0L)
     { /* first type */
       *fa = a_type;
     }
     else
     { /* subseqent type modifier */
       end_of_fa_list->next = a_type;
     }
     end_of_fa_list = a_type;

#if DEC_ANSI_FORM
     ch = s[ci = end_of_grammar_element+1];
     if (ch == 'T' || ch == 'N') {
       if (ch == 'N') {
	 if (s[ci+1] > '9' || s[ci+1] < '1') {
	   _delete_type_list(*fa);
	   *fa = 0L; /* set to zero because nothing found */
	   *end = start - 1;
	   DEMANGLE_TRACEOUT(_function_arguments);  
	   return MLD_NOTHING_RECOGNIZED;
	 }
	 ci++;
       }
       ci++;
       status = _repeat_arg_type(*fa, s, ci, &end_of_grammar_element, &a_type);
       if (status == MLD_ERROR || status == MLD_NOTHING_RECOGNIZED) {
	 _delete_type_list(*fa);
	 *fa = 0L; /* set to zero because nothing found */
	 *end = start - 1;
	 DEMANGLE_TRACEOUT(_function_arguments);  
	 return MLD_NOTHING_RECOGNIZED;
       }
       if (ch == 'N')
	 a_type->repeat_count = s[ci-1] - '0';
     } else 
#endif
       {
       /* Construct a_type */
       /* Limit the string to look at */
       status = _pascal_string(s, end_of_grammar_element+1, 
					&end_of_pascal_string, string, STRSIZ);
       if (ansi_form && status == MLD_SOMETHING_RECOGNIZED) {
	 /* Reconstruct the string from the begining. */

	 strncpy(&string2[0], s, end_of_pascal_string+1);
	 string2[end_of_pascal_string+1] = 0;
	 status = _type(&string2[0], end_of_grammar_element + 1, 
					&end_of_grammar_element, &a_type);
       } else
	 status = _type(s, end_of_grammar_element + 1, 
		      &end_of_grammar_element, &a_type); /* allocs mem */
       }
  }
  
  if (status == MLD_ERROR)
  {
    /* delete existing fa */
    _delete_type_list(*fa);
    
    *fa = 0L;
    *end = start - 1;
    DEMANGLE_TRACEOUT(_function_arguments);  
    return MLD_ERROR;
  }
  
  /* Handle case where nothing is recognized... */  
  if (*fa != 0L) 
  { /* Actually Something Recognized */
    *end = end_of_grammar_element;
    DEMANGLE_TRACEOUT(_function_arguments);  
    return MLD_SOMETHING_RECOGNIZED;
  }
  else
  { /* Nothing Recognized */
    _delete_type_list(*fa);
    *fa = 0L; /* set to zero because nothing found */
    *end = start - 1;
    DEMANGLE_TRACEOUT(_function_arguments);  
    return MLD_NOTHING_RECOGNIZED; /* should be at least one function argument */
  }
}
