/*
*****************************************************************************
**                                                                          *
**                         COPYRIGHT (c) 1991 BY                            *
**             DIGITAL EQUIPMENT CORPORATION, MAYNARD, MASS.                *
**                          ALL RIGHTS RESERVED                             *
**                                                                          *
**  THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND  COPIED  *
**  ONLY  IN  ACCORDANCE  WITH  THE  TERMS  OF  SUCH  LICENSE AND WITH THE  *
**  INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR  ANY  OTHER  *
**  COPIES  THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY  *
**  OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF  THE  SOFTWARE  IS  HEREBY  *
**  TRANSFERRED.                                                            *
**                                                                          *
**  THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE  WITHOUT  NOTICE  *
**  AND  SHOULD  NOT  BE  CONSTRUED  AS  A COMMITMENT BY DIGITAL EQUIPMENT  *
**  CORPORATION.                                                            *
**                                                                          *
**  DIGITAL ASSUMES NO RESPONSIBILITY FOR THE USE OR  RELIABILITY  OF  ITS  *
**  SOFTWARE ON EQUIPMENT WHICH IS NOT SUPPLIED BY DIGITAL.                 *
**                                                                          *
*****************************************************************************
**++
**  PRODUCT:
***
**	DEC C++ Demangler
**
**  FILE:  
**
**      grammer_elements.h
**
**  ABSTRACT:
**
**	Definitions of the grammar elements used in the demangler.
**
**  ENVIRONMENT:
**
**	User Mode
**
**  AUTHORS:
**
** 	Shell S. Simpson (DECwest Engineering)
**
**  MODIFICATION HISTORY:
**
**	06-Dec-1994	Harold Seigel	Change STRSIZE from 100 to 1032.
**	15-Jun-1998	Shi-Jung Kao	Restructed, made closer to the system.
**	13-May-1999	Shi-Jung Kao	
**		Raise STRSIZ from 1032 to 4096, avoid internal overfolw.
**--
**/


#ifndef _GRAMMAR_ELEMENTS_H_
#define _GRAMMAR_ELEMENTS_H_

#ifdef CXX_NT
#include "mn_nm.h"
#else
#include "demangle.h"
#endif

/* define the size of the string used */
#define STRSIZ 4096

/* macros used with long truncated names */
#define MAX_NAME_LENGTH 1022
#define CRC_LENGTH 7
#define TRUNC_LENGTH (MAX_NAME_LENGTH - CRC_LENGTH)


extern long treat_as_trunc;

/* grammar elements */

#if DEC_ANSI_FORM
int /* recognition status */
_address_type(const char* s,	/* IN:  string to be parsed */
	      int start,	/* IN:  index of first unparsed character */
	      int* end,		/* OUT: index of last parsed character */
	      struct MLD_type** t);
				/* OUT: type */
#else
int /* recognition status */
_address_type(const char* s,	/* IN:  string to be parsed */
      		 int start,	/* IN:  index of first unparsed character */
       		 int* end,	/* OUT: index of last parsed character */
       		 char* a,	/* OUT: string to contain address object */
       		 int max_a);	/* IN:	maximum size of a */
#endif

void
_make_address_type(
	   char* addr_name,		/* IN: name */
	   struct MLD_qualification* q, /* IN: qualification. */
	   struct MLD_type** t);	/* OUT: a MLD_type. */

int /* recognition status */
_array_type(const char* s,	/* IN:  string to be parsed */
           int start,		/* IN:  index of first unparsed character */
           int* end,		/* OUT: index of last parsed character */
           struct MLD_array** p);
				/* OUT: structure containing array 
				   (allocated) */

int /* recognition status */
_basic_type(const char* s,	/* IN:  string to be parsed */
           int start,		/* IN:  index of first unparsed character */
       	   int* end,		/* OUT: index of last parsed character */
           int* bt);		/* OUT: integer containing basic type number */

int /* recognition status */
_complex_type(const char* s,	/* IN:  string to be parsed */
            int start,		/* IN:  index of first unparsed character */
            int* end,		/* OUT: index of last parsed character */
            struct MLD_complex** c);/* OUT: complex type */

int /* recognition status */
_constant_type(const char* s,	/* IN:  string to be parsed */
      		  int start,	/* IN:  index of first unparsed character */
       		  int* end,	/* OUT: index of last parsed character */
       		  char* c,	/* OUT: string to contain constant */
       		  int max_c);	/* IN:	maximum size of c */

int /* recognition status */
_constant_type2(const char* s,	/* IN:  string to be parsed */
	       int basic_type,  /* IN:  index of basic type */
	       int start,	/* IN:  index of first unparsed character */
	       int* end,	/* OUT: index of last parsed character */
	       char* c,		/* OUT: string to contain constant */
	       int max_c);	/* IN:	maximum size of c */

int /* TRUE or FALSE */
_is_special_constant_basic_type(struct MLD_type* t, int* id);

int /* recognition status */
_make_constant_type(
             char* c,		/* IN: Constant string */
	     struct MLD_type** tt); /* OUT: MLD_type */

int /* recognition status */
_function_arguments (
	     const char* s,	/* IN:  string to be parsed */
	     int start,		/* IN:  index of first unparsed character */
	     int* end,		/* OUT: index of last parsed character */
	     struct MLD_type** fa);
				/* OUT: function args consist of type list */

int /* recognition status */
_function_type(const char* s,	/* IN:  string to be parsed */
             int start,		/* IN:  index of first unparsed character */
             int* end,		/* OUT: index of last parsed character */
             struct MLD_function** p);
				/* OUT: structure containing function 
				   (allocated) */


int /* recognition status */
_identifier(const char* s,	/* IN:  string to be parsed */
          int start,		/* IN:  index of first unparsed character */
          int* end,		/* OUT: index of last parsed character */
          int num_dbl_uscores,	/* IN:  number of underscores to find */
          char* id,		/* OUT: identifier */
          int id_size,		/* IN:  maximum possible length of id */
          char* before_str,	/* OUT: string coming before identifier */
          int before_str_size);	/* IN:  maximum possible length of before 
				   string */

int /* recognition status */
_number(const char* s,		/* IN:  string to be parsed */
       int start,		/* IN:  index of first unparsed character */
       int* end,		/* OUT: index of last parsed character */
       int* n);			/* OUT: integer value of number recognized */

int /* recognition status */
_pascal_string(const char* s,	/* IN:  string to be parsed */
              int start,	/* IN:  index of first unparsed character */
              int* end,		/* OUT: index of last parsed character */
              char* string,	/* OUT: integer value of number recognized */
              int string_size);	/* IN:  maximum possible length of string */

int /* recognition status */
_pointer_to_member_type(
	const char* s,	/* IN:  string to be parsed */
	int start,	/* IN:  index of first unparsed character */
	int* end,	/* OUT: index of last parsed character */
	struct MLD_pointer_to_member** p);
		/* OUT: structure containing pointer_to_member (allocated) */

int /* recognition status */
_nontype_type(const char* s,	/* IN:  string to be parsed */
	      int is_pointer,   /* IN:  is_pointer, will turn into address */
	      int start,	/* IN:  index of first unparsed character */
	      int* end,		/* OUT: index of last parsed character */
	      struct MLD_type** t);
				/* OUT: type */

int /* recognition status */
_pointer_type(
	const char* s,	/* IN:  string to be parsed */
	int start,	/* IN:  index of first unparsed character */
	int* end,	/* OUT: index of last parsed character */
	struct MLD_pointer** p);
			/* OUT: structure containing pointer (allocated) */

int /* TRUE or FALSE */
_is_pointer_type(struct MLD_type* t);

int /* recognition status */
_parse_qualifications(
        const char* s,	/* IN:  string to be parsed */
	int start,	/* IN:  index of first unparsed character */
	int* end,	/* OUT: index of last parsed character */
	struct MLD_qualification** ql);	
			/* OUT: qualification list (allocates memory) */

int /* recognition status */
_recover_from_error(
        const char* s,	/* IN:  string to be parsed */
	int start,	/* IN:  index of first unparsed character */
	int* end,	/* OUT: index of last parsed character */
	char* bs,	/* OUT: string that gets passed over */
	int bs_max);	/* IN:	maximum size of bs */
                  
int /* recognition status */
_reference_type(
	const char* s,	/* IN:  string to be parsed */
	int start,	/* IN:  index of first unparsed character */
	int* end,	/* OUT: index of last parsed character */
	struct MLD_reference** p);
			/* OUT: structure containing reference (allocated) */
	 
int /* recognition status */
_template_arguments (
        const char* s,	/* IN:  string to be parsed */
	int start,	/* IN:  index of first unparsed character */
	int* end,	/* OUT: index of last parsed character */
	struct MLD_type** ta);
			/* OUT: template args consist of type list */

int /* recognition status */
_template_type(
        const char* s,	/* IN:  string to be parsed */
	int start,	/* IN:  index of first unparsed character */
	int* end,	/* OUT: index of last parsed character */
	struct MLD_type** tt);/* OUT: template type */

int /* recognition status */
_repeat_arg_type(
      struct MLD_type* arg1,	/* IN:  Head of the type list */
      const char* s,		/* IN:  string to be parsed */
      int start,		/* IN:  index of first unparsed character */
      int* end,			/* OUT: index of last parsed character */
      struct MLD_type** t);	/* OUT: type */

int /* recognition status */
_type(const char* s,		/* IN:  string to be parsed */
     int start,			/* IN:  index of first unparsed character */
     int* end,			/* OUT: index of last parsed character */
     struct MLD_type** t);	/* OUT: type */

int /* recognition status */
_type_modifiers
	(const char* s,		/* IN:  string to be parsed */
     int start,			/* IN:  index of first unparsed character */
     int* end,			/* OUT: index of last parsed character */
     struct MLD_type_modifiers** tml);
			       /* OUT: type modifier list (allocates memory) */

int /* recognition status */
_type_modifier(char* s,		/* IN:  string to be parsed */
              int start,	/* IN:  index of first unparsed character */
       	      int* end,		/* OUT: index of last parsed character */
              char* tms,	/* OUT: string containing type modifer */
              int tms_size);    /* IN: maximum possible length of tms string */

int /* recognition status */
_unmodified_type
	(const char* s,		/* IN:  string to be parsed */
	 int start,		/* IN:  index of first unparsed character */
	 int* end,		/* OUT: index of last parsed character */
	 struct MLD_unmodified_type** ut);
				/* OUT: unmodified type pointer */

int /* recognition status */
_white_space(const char* s,	/* IN:  string to be parsed */
            int start,		/* IN:  index of first unparsed character */
            int* end);		/* OUT: index of last parsed character */

int
_mangled_template_name
    (const char* s,		/* IN:  string to be parsed */
     int start,			/* IN:  index of first unparsed character */
     int* end,			/* OUT: index of last parsed character */
     struct MLD_mangled_template_name** mtn,
				/* OUT: struct containing mangled templ name */
     char* str_before_name,	/* OUT: string before mangled templ name */
     char* str_replaced_by_name,
				/* OUT: string replaced by mangled templ name*/
     char* str_after_name,	/* OUT: string after mangled templ name */
     int size_of_string);	/* IN:	max size of the strings listed above */

int
_external_suffix(const char* s,	/* IN:  string to be parsed */
            int start,		/* IN:  index of first unparsed character */
            int* end);		/* OUT: index of last parsed character */

int
_mangled_function_name
    (const char* s,		/* IN:  string to be parsed */
     int start,			/* IN:  index of first unparsed character */
     int* end,			/* OUT: index of last parsed character */
     struct MLD_mangled_function_name** mfn,
				/* OUT: struct containing mangled funct name */
     char* str_before_name,	/* OUT: string before mangled funct name */
     char* str_replaced_by_name,/* OUT: string replaced by mangled funct name*/
     char* str_after_name,	/* OUT: string after mangled funct name */
     int size_of_string);	/* IN:	max size of the strings listed above */

int
_mangled_variable_name
    (const char* s,		/* IN:  string to be parsed */
     int start,			/* IN:  index of first unparsed character */
     int* end,			/* OUT: index of last parsed character */
     struct MLD_mangled_variable_name** mvn,
				/* OUT: struct containing mangled var name */
     char* str_before_name,	/* OUT: string before mangled var name */
     char* str_replaced_by_name,/* OUT: string replaced by mangled var name */
     char* str_after_name,	/* OUT: string after mangled var name */
     int size_of_string);	/* IN:	max size of the strings listed above */

int
_virtual_table_name
    (const char* s,		/* IN:  string to be parsed */
     int start,			/* IN:  index of first unparsed character */
     int* end,			/* OUT: index of last parsed character */
     char* message,		/* OUT: struct containing mangled funct name */
     char* str_before_name,	/* OUT: string before mangled funct name */
     char* str_replaced_by_name,/* OUT: string replaced by mangled funct name*/
     char* str_after_name,	/* OUT: string after mangled funct name */
     int size_of_string,	/* IN:	max size of the strings listed above */
     enum MLD_mangled_name_types *mld_type);	
				/* OUT:	max size of the strings listed above */

#endif /* _GRAMMAR_ELEMENTS_H_ */
/* DON'T ADD ANYTHING AFTER THIS #endif */
