#pragma required_pointer_size long
#ifndef lint
static const char sccsid[] = "@(#)identifier.c	1.1     2/10/94";
#endif
/*
*****************************************************************************
**                                                                          *
**                         COPYRIGHT (c) 1991 BY                            *
**             DIGITAL EQUIPMENT CORPORATION, MAYNARD, MASS.                *
**                          ALL RIGHTS RESERVED                             *
**                                                                          *
**  THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND  COPIED  *
**  ONLY  IN  ACCORDANCE  WITH  THE  TERMS  OF  SUCH  LICENSE AND WITH THE  *
**  INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR  ANY  OTHER  *
**  COPIES  THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY  *
**  OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF  THE  SOFTWARE  IS  HEREBY  *
**  TRANSFERRED.                                                            *
**                                                                          *
**  THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE  WITHOUT  NOTICE  *
**  AND  SHOULD  NOT  BE  CONSTRUED  AS  A COMMITMENT BY DIGITAL EQUIPMENT  *
**  CORPORATION.                                                            *
**                                                                          *
**  DIGITAL ASSUMES NO RESPONSIBILITY FOR THE USE OR  RELIABILITY  OF  ITS  *
**  SOFTWARE ON EQUIPMENT WHICH IS NOT SUPPLIED BY DIGITAL.                 *
**                                                                          *
*****************************************************************************
**++
**  PRODUCT:
***
**	DEC C++ Demangler
**
**  FILE:  
**
**      identifier.c
**
**  ABSTRACT:
**
**	Grammar element that parses a <grammar component>.
**
**  ENVIRONMENT:
**
**	User Mode
**
**  AUTHORS:
**
** 	Shell S. Simpson (DECwest Engineering)
**
**  MODIFICATION HISTORY:
**
**  03-Mar-2000  Shi-Jung Kao
**	Skip version7 signature from both id and bs.
**	Added DEMANGLE_ASSERT.
**
**--
**/


#ifdef CXX_NT
#include "defs.h"
#include "mn_nm.h"
#include "gram_elm.h"
#include "util.h"
#else
#include "demangle.h"
#include "grammar_elements.h"
#endif

#include "demangle_internal.h"

#include <stdlib.h>
#include <stddef.h>
#include <stdio.h>
#include <string.h>

/* language element particular definitions */
#define ASCII_ZERO 48

int /* recognition status */
_identifier(const char* s,		/* IN:  string to be parsed */
          int start,			/* IN:  index of first unparsed character */
          int* end,				/* OUT: index of last parsed character */
          int num_dbl_uscores,	/* IN:  number of underscores to find */
          char* id,				/* OUT: identifier */
          int id_size,			/* IN:  maximum possible length of id */
          char* before_str,		/* OUT: string coming before identifier */
          int before_str_size)  /* IN:  maximum possible length of before string */
{
  /* variables common to all language elements */
  int ci;  /* current index */
  char ch; /* current character */
  
  /* variables particular to this language element */
  int number_of_chars_in_ident;
  int start_position_of_ident;
  int number_of_consecutive_underscores;
  char sub[STRSIZ];
  int status;
  int end_of_grammar_component;
  struct MLD_type* cot;
  int current_number_of_dunderscores = 0; /* initially */

  /* initialization particular to language element */
  number_of_chars_in_ident = 0;
  start_position_of_ident = 0;
  number_of_consecutive_underscores = 0;

  *end = start - 1;

  ch = s[ci = start]; /* initialize ch and ci */
  while (ch != 0)
  {
    /* handle "__op" */
    if (strncmp(&s[ci], OPERATOR_CONVERSION_TOKEN, 4) == 0)
    {
      cot = 0;
      status = _type(s, ci+4, &end_of_grammar_component, &cot);
      _delete_type_list(cot);

      if (status == MLD_SOMETHING_RECOGNIZED)
      {
        start_position_of_ident = ci;
        number_of_chars_in_ident =   end_of_grammar_component 
                                   - start_position_of_ident + 1;

        /* mark end of parsed portion of string */
        *end = end_of_grammar_component; /* don't include the two underscores */

        /*
        ** copy identifier
        */
        if (number_of_chars_in_ident < id_size) /* if big enough... */
        { /* ...copy into id */
          strncpy(id, &s[start_position_of_ident], 
                  number_of_chars_in_ident);
          id[number_of_chars_in_ident] = 0; /* null terminate */
        }
        else /* not id not big enough--Error! */
        { 
          *end = start - 1;
          return MLD_ERROR;
        }
        
        /*
        ** copy text coming before identifier
        */
        if (start_position_of_ident != 0)
        {
          if ((start_position_of_ident-start) < before_str_size) /* if big enough... */
          {
            strncpy(before_str, &s[start], start_position_of_ident - start);
            before_str[start_position_of_ident - start] = 0; /* null terminate */
          }
          else /* not before string not big enough--Error! */
          { 
            *end = start - 1;
            return MLD_ERROR;
          }
        }
        else
        {
          if (before_str_size > 0) /* if big enough... */
          {
            strcpy(before_str, "");
          }
          else /* not before string not big enough--Error! */
          { 
            *end = start - 1;
            return MLD_ERROR;
          }
        }
      }
      
      return MLD_SOMETHING_RECOGNIZED;
    } 

    /* handle underscore */
    if (ch == '_')
    {
      if (number_of_chars_in_ident == 0)
      {
         start_position_of_ident = ci; 
      }

      number_of_chars_in_ident++;

      if (number_of_chars_in_ident > 0)
      {
        /* add one to the underscore count */
        number_of_consecutive_underscores++; 
      }

      if ((number_of_consecutive_underscores > 1) &&
          (number_of_chars_in_ident > 2))
      {
        /* a full identifier has been recognized... */
        
        /* ...but maybe there are additional underscores! */
        while (s[ci+1] == '_')
        {
          ch = s[++ci]; /* increment ch and ci */
        }
        
        /* we've recognized a double underscore, add to the count */
        current_number_of_dunderscores++;
        
        if (current_number_of_dunderscores == num_dbl_uscores)
        {
          /* mark end of parsed portion of string */
          *end = ci - 2; /* don't include the two underscores */

          /*
          ** copy identifier
          */
          if ((ci-start_position_of_ident-1) < id_size) /* if big enough... */
          { /* ...copy into id */
            strncpy(id, &s[start_position_of_ident], 
                    ci-start_position_of_ident-1);
            id[ci-start_position_of_ident-1] = 0; /* null terminate */
          }
          else /* not id not big enough--Error! */
          { 
            *end = start - 1;
            return MLD_ERROR;
          }
        
          /*
          ** copy text coming before identifier
          */
          if (start_position_of_ident != 0)
          {
            if (start_position_of_ident < before_str_size) /* if big enough... */
            {
	      /* Do not put version signature back to bs either. */

	      int version7_signature = 0;
	      if (start_position_of_ident - start >= VERSION_STRING_LENGTH &&
		  strncmp (&s[start_position_of_ident-VERSION_STRING_LENGTH],
			   VERSION7, VERSION_STRING_LENGTH) == 0)
		version7_signature = VERSION_STRING_LENGTH;
              strncpy(before_str, &s[start], 
		      start_position_of_ident - start - version7_signature);

	      /* null terminate */
              before_str[start_position_of_ident - start -version7_signature] = 0;
            }
            else /* not before string not big enough--Error! */
            { 
              *end = start - 1;
              return MLD_ERROR;
            }
          }
          else
          {
            if (before_str_size > 0) /* if big enough... */
            {
              strcpy(before_str, "");
            }
            else /* not before string not big enough--Error! */
            { 
              *end = start - 1;
              return MLD_ERROR;
            }
          }

          return MLD_SOMETHING_RECOGNIZED;
        }
      }
      
      ch = s[++ci]; /* increment ch and ci */
      continue;
    }

    /* handle letter */
    if ( ((ch >= 'A') && (ch <= 'Z')) || (ch == '$') ||
         ((ch >= 'a') && (ch <= 'z')) )
    {
      if (number_of_chars_in_ident == 0)
      {
         start_position_of_ident = ci;
      }

      number_of_consecutive_underscores = 0;
      number_of_chars_in_ident++;

      ch = s[++ci]; /* increment ch and ci */
      continue;
    }

    /* handle number */
    if ((ch >= '0') && (ch <= '9'))
    {
      number_of_consecutive_underscores = 0;
      number_of_chars_in_ident++;
  
      if (number_of_chars_in_ident < 2)
      {
         number_of_chars_in_ident = 0;
 
         /* eat all the characters until a white space is encountered */
         while( !( (ch == 32)   || /* is it a space */
                   (ch == 10)   || /* is it a line feed */
                   (ch == 9)    || /* is it a tab */
                   (ch == 13) ) )  /* is it a return */
         {
            ch = s[++ci]; /* increment ch and ci */
         }

         continue;
      }

      ch = s[++ci]; /* increment ch and ci */
      continue;
    }

    /* handle "other" */
    /* Skip the version signature, if it ever got to here. */
    if (ch == ' ' && 
	strncmp (&s[ci+1], VERSION7, VERSION_STRING_LENGTH) == 0) {
      DEMANGLE_ASSERT(MLD_mangled_version == MLD_VERSION7,
		      Demangle_unexpected_version7_signature);
      ci += VERSION_STRING_LENGTH;
    }
    number_of_consecutive_underscores = 0;
    number_of_chars_in_ident = 0;
    start_position_of_ident = 0;

    ch = s[++ci]; /* increment ch and ci */
    continue;
  } /* end of while loop */

  /* loop completed, nothing found at this point */
  *end = start - 1;

  /* all text between start and end in s comes "before" the recognized identifiier */
  if ((strlen(s)-start) < before_str_size) /* if big enough... */
  {
    strcpy(before_str, &s[start]);
  }
  else
  {
    return MLD_ERROR;
  }
  
  return MLD_NOTHING_RECOGNIZED;  
}
