#pragma required_pointer_size long
#ifndef lint
static const char sccsid[] = "@(#)mangled_function_name.c	1.1     2/10/94";
#endif
/*
*****************************************************************************
**                                                                          *
**                         COPYRIGHT (c) 1991, 1999 BY                      *
**             DIGITAL EQUIPMENT CORPORATION, MAYNARD, MASS.                *
**                          ALL RIGHTS RESERVED                             *
**                                                                          *
**  THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND  COPIED  *
**  ONLY  IN  ACCORDANCE  WITH  THE  TERMS  OF  SUCH  LICENSE AND WITH THE  *
**  INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR  ANY  OTHER  *
**  COPIES  THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY  *
**  OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF  THE  SOFTWARE  IS  HEREBY  *
**  TRANSFERRED.                                                            *
**                                                                          *
**  THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE  WITHOUT  NOTICE  *
**  AND  SHOULD  NOT  BE  CONSTRUED  AS  A COMMITMENT BY DIGITAL EQUIPMENT  *
**  CORPORATION.                                                            *
**                                                                          *
**  DIGITAL ASSUMES NO RESPONSIBILITY FOR THE USE OR  RELIABILITY  OF  ITS  *
**  SOFTWARE ON EQUIPMENT WHICH IS NOT SUPPLIED BY DIGITAL.                 *
**                                                                          *
*****************************************************************************
**++
**  PRODUCT:
***
**	DEC C++ Demangler
**
**  FILE:  
**
**      mangled_name.c
**
**  ABSTRACT:
**
**	Grammar element that parses a mangled_name.
**
**  ENVIRONMENT:
**
**	User Mode
**
**  AUTHORS:
**
** 	Shell S. Simpson (DECwest Engineering)
**
**  MODIFICATION HISTORY:
**
**      03-Apr-1995	Harold Seigel
**      Fix memory leak.  CXXC_BUGS 2550
**
**      09-Jul-1996	Harold Seigel
**      Check lengths before calling strcpy to prevent overwriting the stack.
**
**	19-Feb-1998	Shi-Jung Kao
**	_delete_mangled_function_name before return ERROR. (free is insufficient.)
**
**	03-Apr-1998	Shi-Jung Kao
**	_delete_type_list(a) 2 places to free all mallocs.
**
**      18-May-1999     PremAnand M. Rao
**      Commented a portion of code which was causing
**      "unreachable" warnings to be issued.
**
**      19-May-1999     PremAnand M. Rao
**      Shi-Jung verified the commented portion and I am
**      removing the printf message from there.
**
**      19-May-1999     PremAnand M. Rao
**      Previous edit was incomplete, the loop in map_nontype_ut()
**      should complete, and not return after its first iteration
**      as before.
**
**      30-Jun-1999     Shi-Jung Kao
**      _delete rtn_type and ftml from previous loop if any.
**
**      22-Oct-1999     Shi-Jung Kao
**      Accept new ANSI_FUNCTION_TOKEN.
**
**	27-Oct-1999	Shi-Jung Kao
**	Fill in code for MLD_FUNCTION_TYPE in map_nontype_.. for new rules.
**
**	18-Nov-1999	Shi-Jung Kao
**	Support __S in function names. (specialized)
**
**	22-Nov-1999	Shi-Jung Kao
**	Set shared_id for nontype. (i.e. <T1=int...>)
**
**	20-Jan-2000	Shi-Jung Kao
**	Under version7, accept nontype in conversion_operator_type.
**	e.g. __7__opZ1Z__tm__2_i__1CFv_Z1Z
**
**	25-Jan-2000	Shi-Jung Kao
**	Stop the demangle when reachs '\0'.
**	Added _cleanup_onerror.
**
**	07-Feb-2000	Shi-Jung Kao
**	Accepted friend specialized function "__S" before FUNCTION_TOKEN.
**	Processed array_type.nontype_dim. i.e. nontype dimension.
**	Processed nested nontype.
**	Now, model ansi emits "const" after class qualifier for member
**	functions. (ARM emits before.)
**	Added DEMANGLE_TRACEIN/TRACEOUT.
**	Accept static member function "S" under -model ansi.
**
**--
**/


#ifdef CXX_NT
#include "defs.h"
#include "mn_nm.h"
#include "gram_elm.h"
#include "util.h"
#else
#include "demangle.h"
#include "grammar_elements.h"
#endif

#include "demangle_internal.h"

#include <stdlib.h>
#include <stddef.h>
#include <stdio.h>
#include <string.h>

/* language element particular definitions */
#define ASCII_ZERO 48

static
struct MLD_qualification* get_mz_qual(/* Zn_mZ: get qualicication at level m */
	 struct MLD_qualification* q, /* IN: May contain template args. */
	 int level);		      /* IN: level of the qualification */
                                      /* Return the qualification of level. */
static
void map_nontype(
	 struct MLD_qualification* q, /* IN: May contain template args. */
	 struct MLD_type** type);     /* IN/OUT: nontype to be mapped */
static
void map_nontype_ut(
	 struct MLD_qualification* q, /* IN: May contain template args. */
	 struct MLD_unmodified_type* type); /* IN/OUT: nontype to be mapped */

static
void _cleanup_onerror (struct MLD_type_modifiers* ftml,
		       struct MLD_qualification* q,
		       struct MLD_type* a,
		       struct MLD_type* rtn_type);

int /* recognition status */
_mangled_function_name
    (const char* s,		/* IN:  string to be parsed */
     int start,			/* IN:  index of first unparsed character */
     int* end,			/* OUT: index of last parsed character */
     struct MLD_mangled_function_name** mfn,
				/* OUT: struct containing mangled funct name */
     char* str_before_name,	/* OUT: string before mangled funct name */
     char* str_replaced_by_name,/* OUT: string replaced by mangled funct name*/
     char* str_after_name,	/* OUT: string after mangled funct name */
     int size_of_string)	/* IN:	max size of the strings listed above */
{
  /* variables common to all language elements */
  int ci;  /* current index */
  char ch; /* current character */
  
  /* variables particular to this language element */
  int status;
  int end_of_grammar_component;
  struct MLD_mangled_function_name* temp_mn;
  char bs[STRSIZ];		/* string before demangled string */
  char rs[STRSIZ];		/* string replaced by demangled string */
  char as[STRSIZ];		/* string after demangled string */
  char id[STRSIZ];		/* identifier string */
  char string[STRSIZ];		/* generic string for temporary use */
  struct MLD_qualification* q=0;/* qualifier */
  struct MLD_complex* class;	/* class */
  struct MLD_type* a=0;		/* function argument */
  struct MLD_type* rtn_type=0;	/* function return */
  int end_of_mn;		/* index of last character in s of mn */
  struct MLD_type_modifiers* ftml=0;	/* function type modifier list */
  struct MLD_type* cot;			/* converstion operator type */
  enum MLD_function_name_types fnt;	/* constructor, destructor, etc... */

  int num_du = 0;		/* number of double underscores */
  int du_count;                 /* looping variable that determines the
                                   underscore number that will be used
                                   in the identifier */
  
  DEMANGLE_TRACEIN(_mangled_function_name);

  num_du = _num_dbl_uscores(s, start);

  for (du_count=1; du_count<=num_du; du_count++)
  { /* start of loop that repeatled check for double underscores */
    

  /* initialization particular to language element */
  temp_mn = 0L;
  *mfn = 0L;
  
  /*
  **
  ** Get the identifier
  **
  */

  ch = s[ci = start]; /* initialize ci and ch */
  status = _identifier(s, ci, &end_of_grammar_component, du_count,
                                           id, STRSIZ, bs, STRSIZ);
  switch (status)
  {    
    case MLD_NOTHING_RECOGNIZED:
      continue;
    break;
    
    case MLD_ERROR:
      /* if error occured, return ERROR */
      *end = start - 1; /* nothing recognized */
      *mfn = 0L; /* no mangled name */
      
      if (size_of_string > 0) /* copy if enough room */
      {
        strcpy(str_before_name,"");
        strcpy(str_replaced_by_name,"");
      }
      
      if ((strlen(s) - start) < size_of_string) /* copy if enough room */
      {
        strcpy(str_after_name, &s[start]); /* everything is "after" */
      }
      _cleanup_onerror (ftml, q, a, rtn_type);
      DEMANGLE_TRACEOUT(_mangled_function_name);
      return MLD_ERROR;
    break;
  }
  ch = s[ci = (end_of_grammar_component+1)]; /* update ci and ch */

  if ((s[ci] == '_') && (s[ci+1] == '_')) {
    /*
    **
    ** Get the "__"
    **
    */
    ch = s[ci = (ci+2)]; /* update ci and ch */

    /*
    **
    ** Get the "__S" if the template from which this function is generated 
    ** is specialized.
    **
    */
    if (s[ci] == SPECIAL_TOKEN) {
      end_of_grammar_component = end_of_grammar_component + 3;
      ch = s[++ci];
    }
  } else
    continue;

  if (ftml != 0) {
      DEMANGLE_FREE(ftml);
      ftml = 0;
  }

  /*
  **
  ** Get the type modifiers to the function
  **
  */

  status = _type_modifiers(s, ci, &end_of_grammar_component, &ftml);
  if ((status == MLD_SOMETHING_RECOGNIZED) && 
      ((ftml->signed_tm) || (ftml->unsigned_tm)))
  {
    continue;
  }
  
  if (status == MLD_ERROR)
  {
    /* if error occured, return ERROR */
    *end = start - 1; /* nothing recognized */
    *mfn = 0L; /* no mangled name */
      
    if (size_of_string > 0) /* copy if enough room */
    {
      strcpy(str_before_name,"");
      strcpy(str_replaced_by_name,"");
    }
 
    if ((strlen(s) - start) < size_of_string) /* copy if enough room */
    {
      strcpy(str_after_name, &s[start]); /* everything is "after" */
    }
    _cleanup_onerror (ftml, q, a, rtn_type);
    DEMANGLE_TRACEOUT(_mangled_function_name);
    return MLD_ERROR;
  }
  ch = s[ci = (end_of_grammar_component+1)]; /* update ci and ch */


  /*
  ** deallocate memory allocated by a possible previous call to
  ** _parse_qualifications().
  */
  _delete_qualifications(q);
  q=0;

  /*
  **
  ** Get the qualifications
  **
  */
  status = _parse_qualifications(s, ci, &end_of_grammar_component, &q);
 
  switch (status)
  {    
    case MLD_NOTHING_RECOGNIZED:
      /* no problem */
      q = 0L; /* make sure that qualification list is empty */
      break;
    
    case MLD_ERROR:
      /* if error occured, return ERROR */
      *end = start - 1; /* nothing recognized */
      *mfn = 0L; /* no mangled name */
      
      if (size_of_string > 0) /* copy if enough room */
      {
        strcpy(str_before_name,"");
        strcpy(str_replaced_by_name,"");
      }
      
      if ((strlen(s) - start) < size_of_string) /* copy if enough room */
      {
        strcpy(str_after_name, &s[start]); /* everything is "after" */
      }
      _cleanup_onerror (ftml, q, a, rtn_type);
      DEMANGLE_TRACEOUT(_mangled_function_name);
      return MLD_ERROR;
      break;

    default:
      break;
  }
  ch = s[ci = (end_of_grammar_component+1)]; /* update ci and ch */

  /*
  ** Under model ansi, the "const" is after the class qualifier.
  ** Get the type modifiers to the function
  ** Limit to the CONST_TOKEN, otherwise interfer with __S.
  **
  */

  if (MLD_mangled_version != MLD_PRE_VERSION7 && ftml == 0 &&
      ch == CONST_TOKEN) {
    status = _type_modifiers(s, ci, &end_of_grammar_component, &ftml);
    if ((status == MLD_SOMETHING_RECOGNIZED) && 
	((ftml->signed_tm) || (ftml->unsigned_tm)))
      {
	continue;
      }
  
    if (status == MLD_ERROR)
      {
	/* if error occured, return ERROR */
	*end = start - 1; /* nothing recognized */
	*mfn = 0L; /* no mangled name */
	
	if (size_of_string > 0) /* copy if enough room */
	  {
	    strcpy(str_before_name,"");
	    strcpy(str_replaced_by_name,"");
	  }
	
	if ((strlen(s) - start) < size_of_string) /* copy if enough room */
	  {
	    strcpy(str_after_name, &s[start]); /* everything is "after" */
	  }
	_cleanup_onerror (ftml, q, a, rtn_type);
	DEMANGLE_TRACEOUT(_mangled_function_name);
	return MLD_ERROR;
      }
    ch = s[ci = (end_of_grammar_component+1)]; /* update ci and ch */
  }

  /* Break when hits end of the string. */
  if (ch == '\0')
    break;

  /*
  **
  ** Get the "__S" if the template from which this function is generated 
  ** is specialized -- a friend function.
  ** Note, the parse_qualifications takes preceding this one.
  ** i.e. f__tm__2_v__S1SFv_v
  **       ^^^^^^^^^^^^^^ <===== will be parsed upto this point at
  **	previous call to _parse_qualifications.
  ** Or a static member function, just "S". e.g. handle_foo__9foo_classSFPv
  ** cxxc_bugs:7638
  **
  */
  if (ch == SPECIAL_TOKEN /*&& s[ci-1] == '_' && s[ci-2] == '_'*/) {
    if (s[ci+1] == '_' && s[ci+2] == '_') {
      end_of_grammar_component = end_of_grammar_component + 3;
      ch = s[ci = (end_of_grammar_component+1)];
    } else
      ch = s[++ci];
  }

  /*
  **
  ** Get the FUNCTION_TOKEN ('X'--originally)
  **
  */
  
  if (!(ch == FUNCTION_TOKEN || ch == ANSI_FUNCTION_TOKEN))
  {
    continue;
  }
  ch = s[ci = (ci+1)]; /* update ci and ch */
  

  /*
  ** deallocate memory allocated by a possible previous call to
  ** _function_arguments().
  */
  _delete_type_list(a);
  a=0;

  /*
  **
  ** Get the function arguments
  **
  */

  status = _function_arguments(s, ci, &end_of_grammar_component, &a);
  
  switch (status) {    
  case MLD_NOTHING_RECOGNIZED:
    continue;
    break;
    
  case MLD_ERROR:
      /* if error occured, return ERROR */
      *end = start - 1; /* nothing recognized */
      *mfn = 0L; /* no mangled name */
      
      if (size_of_string > 0) /* copy if enough room */
      {
        strcpy(str_before_name,"");
        strcpy(str_replaced_by_name,"");
      }
      
      if ((strlen(s) - start) < size_of_string) /* copy if enough room */
      {
        strcpy(str_after_name, &s[start]); /* everything is "after" */
      }
      _cleanup_onerror (ftml, q, a, rtn_type);
      DEMANGLE_TRACEOUT(_mangled_function_name);
      return MLD_ERROR;
    break;

  default:
    map_nontype(q, &a);
    break;
  }
  ch = s[ci = (end_of_grammar_component+1)]; /* update ci and ch */

#if DEC_ANSI_FORM
  _delete_type_structure(rtn_type);
  rtn_type = 0;

  /*
  **
  ** Get the function return type
  **
  */
  if (ch == '_') {
    status = _type(s, ci+1, &end_of_grammar_component, &rtn_type);

    switch (status) {    
    case MLD_NOTHING_RECOGNIZED:
      continue;
      break;
    
    case MLD_ERROR:
      /* if error occured, return ERROR */
      *end = start - 1; /* nothing recognized */
      *mfn = 0L; /* no mangled name */
      
      if (size_of_string > 0) { /* copy if enough room */
        strcpy(str_before_name,"");
        strcpy(str_replaced_by_name,"");
      }
      
      if ((strlen(s) - start) < size_of_string) {/* copy if enough room */
        strcpy(str_after_name, &s[start]); /* everything is "after" */
      }
      _cleanup_onerror (ftml, q, a, rtn_type);
      DEMANGLE_TRACEOUT(_mangled_function_name);
      return MLD_ERROR;
      break;    

    default:
      map_nontype(q, &rtn_type);
      break;
    }
    ch = s[ci = (end_of_grammar_component+1)]; /* update ci and ch */
  }
#endif

  /*
  **
  ** Check if there is an external suffix, if there is discard it
  **
  */
  if (strlen(s) > end_of_grammar_component+1) {
    status = _external_suffix(s, end_of_grammar_component+1, 
					&end_of_grammar_component);

    switch (status) {
    case MLD_NOTHING_RECOGNIZED: /* OK not to recognize */
      break;

    case MLD_ERROR:
      /* if error occured, return ERROR */
      *end = start - 1; /* nothing recognized */
      *mfn = 0L; /* no mangled name */

      if (size_of_string > 0) /* copy if enough room */
      {
        strcpy(str_before_name,"");
        strcpy(str_replaced_by_name,"");
      }

      if ((strlen(s) - start) < size_of_string) /* copy if enough room */
      {
        strcpy(str_after_name, &s[start]); /* everything is "after" */
      }
      _cleanup_onerror (ftml, q, a, rtn_type);
      DEMANGLE_TRACEOUT(_mangled_function_name);
      return MLD_ERROR;
      break;
    }
    ch = s[ci = (end_of_grammar_component+1)]; /* update ci and ch */
  }

  /*
  **
  ** Check for white space
  **
  */
  
  end_of_mn = end_of_grammar_component;
  if (strlen(s) > end_of_grammar_component+1) {
    status = _white_space(s, end_of_grammar_component + 1, 
					&end_of_grammar_component);
 
    switch (status) {    
    case MLD_NOTHING_RECOGNIZED:
      continue;
    
    case MLD_ERROR:
      /* if error occured, return ERROR */
      
      *end = start - 1; /* nothing recognized */
      *mfn = 0L; /* no mangled name */
      
      if (size_of_string > 0) /* copy if enough room */
      {
        strcpy(str_before_name,"");
        strcpy(str_replaced_by_name,"");
      }
      
      if ((strlen(s) - start) < size_of_string) /* copy if enough room */
      {
        strcpy(str_after_name, &s[start]); /* everything is "after" */
      }
      _cleanup_onerror (ftml, q, a, rtn_type);
      DEMANGLE_TRACEOUT(_mangled_function_name);
      return MLD_ERROR;
      break;
    }
  }
 
  /*
  **
  ** All the trials have been passed.  Now is the type to create the
  ** mangled name structure and set it appropriately.
  **
  */

  /* assume that the function name type is normal */
  fnt = MLD_NORMAL_FUNCTION_NAME;
  
  /* constructor (uses status returned from get_class) */
  if ((strcmp(id, CONSTRUCTOR_TOKEN) == 0) && 
		(status == MLD_SOMETHING_RECOGNIZED))
  {
      fnt = MLD_CONSTRUCTOR_FUNCTION_NAME;
  }
    
  /* destructor (uses status returned from get_class) */
  if ((strcmp(id, DESTRUCTOR_TOKEN) == 0) && 
		(status == MLD_SOMETHING_RECOGNIZED))
  {
      fnt = MLD_DESTRUCTOR_FUNCTION_NAME;
  }

  /* operator conversion (compare first four characters) */
  if (strncmp(id, OPERATOR_CONVERSION_TOKEN, 4) == 0)
  {
    status = _type(id, 4, &end_of_grammar_component, &cot);
    if (status == MLD_SOMETHING_RECOGNIZED)
    {
      fnt = MLD_CONVERSION_OPERATOR_FUNCTION_NAME;
    }
    /* ignore errors */
    /* (Actually, if there's an error, we'll treat the id as a normal id */
  } 
    
  /* check if identifier has a substitute, if so, copy */  
  _basic_operator_substitute(id, string, STRSIZ);
  if (string[0] != 0) /* borrow class variable */
  {
      fnt = MLD_OPERATOR_FUNCTION_NAME;
  }
  
  /* set up structure */
  temp_mn = (struct MLD_mangled_function_name*) 
		malloc(sizeof(struct MLD_mangled_function_name));
  DEMANGLE_RECORD_ALLOC(temp_mn);
  if (temp_mn == 0L) {
    _cleanup_onerror (ftml, q, a, rtn_type);
    DEMANGLE_TRACEOUT(_mangled_function_name);
    return MLD_ERROR; /* error if no memory allocated */
  }

  temp_mn->function_name_type = fnt; /* function type */
  temp_mn->args = a; /* arguments */
  temp_mn->tm_list = ftml; /* type modifiers */
  temp_mn->rtn = rtn_type; /* return type */
  temp_mn->qualifications  = q;

  if (fnt == MLD_CONVERSION_OPERATOR_FUNCTION_NAME)
  {
    if (q)
      map_nontype(q, &cot);
    temp_mn->id.conversion_operator_type = cot;
  }
  else
  {
    temp_mn->id.identifier = (char*) malloc(strlen(id)+1);
    DEMANGLE_RECORD_ALLOC(temp_mn->id.identifier);
    if (temp_mn->id.identifier == 0L)
    {
      if (ftml != 0) {
	  DEMANGLE_FREE(ftml);
      }
      _delete_mangled_function_name(temp_mn);
      DEMANGLE_TRACEOUT(_mangled_function_name);
      return MLD_ERROR; /* error if no memory allocated */
    }
    strcpy(temp_mn->id.identifier, id);
  }  
      
  if (strlen(bs) < size_of_string) /* copy if enough room */
  {
    strcpy(str_before_name, bs);
  }
      
  if ((end_of_mn - (strlen(bs) + start) + 1) < size_of_string)  {
    /* copy if enough room */
    strncpy(str_replaced_by_name,
            &s[start + strlen(bs)],
            end_of_mn - (strlen(bs) + start) + 1 );
    /* null terminate */
    str_replaced_by_name[(end_of_mn - (strlen(bs) + start) + 1)] = 0;
  }
      
  if (strlen(&s[end_of_mn+1]) < size_of_string) /* copy if enough room */
  {
    strcpy(str_after_name, &s[end_of_mn+1]);
  }
      
  *mfn = temp_mn;   
  *end = end_of_mn; /* end of mangled name */
  DEMANGLE_TRACEOUT(_mangled_function_name);
  return MLD_SOMETHING_RECOGNIZED;
  
  } /* end of loop that repeatedly checks for double underscores */
  

  /* deallocate memory allocated by function _parse_qualifications() */
  _delete_qualifications(q);
  q = 0L;

  /*
  ** deallocate memory allocated by a possible previous call to
  ** _function_arguments().
  */
  _delete_type_list(a);
  a=0;

  if (ftml != 0) {
      DEMANGLE_FREE(ftml);
  }
  /* if no identifier was recognized, return NOTHING_RECOGNIZED */
  *end = start - 1; /* nothing recognized */
  *mfn = 0L; /* no mangled name */
      
  if ((strlen(s) - start) < size_of_string) /* copy if enough room */
  {
    strcpy(str_before_name, &s[start]); 
  }
      
  if (size_of_string > 0) /* copy if enough room */
  {
    strcpy(str_replaced_by_name,"");
    strcpy(str_after_name,"");
  }
      
  DEMANGLE_TRACEOUT(_mangled_function_name);
  return MLD_NOTHING_RECOGNIZED;  
}

static
void map_nontype(
	 struct MLD_qualification* q, /* IN: May contain template args. */
	 struct MLD_type** type)      /* IN/OUT: nontype to be mapped */
{
  /* Map the MLD_nontype, i.e. Znz or Zn_mZ to its template argument type. */

  int status;
  char* nontype_str;
  struct MLD_unmodified_type* ut;
  struct MLD_mangled_template_name* mtn;
  struct MLD_type* arg;
  struct MLD_type* nt;
  struct MLD_type* rt;  /* Replaced type */
  int end_of_number;
  int id,level;
  int i;
  struct MLD_qualification* nq = q; /* Nested q */
  int nested_nontype = 0;

  if (!q || !(q->q) || !*type)
    return;
  if (!(q->q->mangled_template_name)) {
    for (nq = q->next; nq; nq = nq->next) {
      if (nq->q->mangled_template_name)
	break;
    }
    if (!nq || !(nq->q->mangled_template_name))
      return;
  }

  for (nt = *type; nt; nt = nt->next) {

    struct MLD_qualification* tq = nq;

    ut = nt->ut;
    rt = nt;
    if (!ut) continue;

    switch (ut->type_of_type) {
    case MLD_NO_TYPE:
    case MLD_BASIC_TYPE:
      continue;

    case MLD_COMPLEX_TYPE:
    case MLD_POINTER_TYPE:
    case MLD_REFERENCE_TYPE:
    case MLD_ARRAY_TYPE:
      map_nontype_ut(tq, ut);
      continue;

    case MLD_FUNCTION_TYPE:
      map_nontype_ut(tq, ut);
      continue;

    case MLD_POINTER_TO_MEMBER_TYPE:
      map_nontype_ut(tq, ut);
      continue;

    case MLD_CONSTANT_TYPE:
    case MLD_ADDRESS_TYPE:
      continue;

    case MLD_NONTYPE_TYPE:
      if (ut->the_type.nontype->nested_args) {
	/* nested_args is a mangled_template_name. */
	map_nontype(q, &(ut->the_type.nontype->nested_args->args));
	nested_nontype = 1;
      }
      break;

    default:
      continue;
    }

    nontype_str = ut->the_type.nontype->nontype_name;
    status = _number(nontype_str, 1, &end_of_number, &id);
    if (status != MLD_SOMETHING_RECOGNIZED)
      continue;
    i = end_of_number + 1;
    level = 1;
    if (strstr(nontype_str, "_") && nontype_str[i] == '_')
      status = _number(nontype_str, i+1, &end_of_number, &level);
    if (status != MLD_SOMETHING_RECOGNIZED)
      return;
    if (level > 1)
      tq = get_mz_qual(tq, level);


    arg = tq->q->name.template_name->args;
    for (i = 1; i < id && arg && arg->next; i++)
      arg = arg->next;
    if (i < id)
      return;
    if (nested_nontype)
      ut->the_type.nontype->ut = arg->ut;
    else {
      _delete_unmodified_type(rt->ut);
      rt->ut = arg->ut;
      if (rt->shared_ut)
      printf("Please report a demangle internal error <shared_ut unexpected TRUE>\n");
      rt->shared_ut = TRUE;
    }
    _set_shared_id(arg->ut);

  }
  
} /* map_nontype */

static
void map_nontype_ut(
	 struct MLD_qualification* q, /* IN: May contain template args. */
	 struct MLD_unmodified_type* ut) /* IN/OUT: nontype to be mapped */
{
  /* Map the MLD_unmodified_type, note, ut has to be a non-nontype.
     i.e. Znz or Zn_mZ to its template argument type. */

  int status;
  char* nontype_str;
  struct MLD_mangled_template_name* mtn;
  struct MLD_type* arg;
  struct MLD_type* nt;
  struct MLD_type* rt = 0L;  /* Replaced type */
  int end_of_number;
  int id,level;
  int i;

  if (!q || !(q->q) || !ut || !(q->q->mangled_template_name))
    return;

  switch (ut->type_of_type) {
  case MLD_NO_TYPE:
  case MLD_BASIC_TYPE:
    return;

  case MLD_COMPLEX_TYPE: {
    struct MLD_qualification* cq;
    for (cq = ut->the_type.complex; cq; cq = cq->next) {
      struct MLD_complex* cplx = cq->q;
      if (cplx->mangled_template_name) {
	struct MLD_mangled_template_name* mtn = cplx->name.template_name;
	struct MLD_type* arg;
	for (arg = mtn->args; arg; arg = arg->next)
	  map_nontype(q, &arg);
      }
    }
    return;
  }

  case MLD_POINTER_TYPE: {
    struct MLD_pointer* ptr = ut->the_type.pointer;
    if (!(ptr->type_modified->ut) || 
	ptr->type_modified->ut->type_of_type != MLD_NONTYPE_TYPE) {
      map_nontype_ut(q, ptr->type_modified->ut);
      return;
    }
    ut = ptr->type_modified->ut;
    rt = ptr->type_modified;
    break;
  }
  case MLD_REFERENCE_TYPE: {
    struct MLD_reference* ref = ut->the_type.reference;
    if (!(ref->type_modified->ut) || 
	ref->type_modified->ut->type_of_type != MLD_NONTYPE_TYPE) {
      map_nontype_ut(q, ref->type_modified->ut);
      return;
    }
    ut = ref->type_modified->ut;
    rt = ref->type_modified;
    break;
  }
  case MLD_ARRAY_TYPE: {
    struct MLD_array* array = ut->the_type.array;

    if (array->nontype_dim && array->nontype_dim->ut)
      map_nontype(q, &(array->nontype_dim));

    if (!(array->type_modified->ut) || 
	array->type_modified->ut->type_of_type != MLD_NONTYPE_TYPE) {
      map_nontype_ut(q, array->type_modified->ut);
      return;
    }
    ut = array->type_modified->ut;
    rt = array->type_modified;
    break;
  }

  case MLD_FUNCTION_TYPE: {
    struct MLD_type* arg;
    struct MLD_type* rtn;
    struct MLD_function* f = ut->the_type.function;

    for (arg = f->args; arg; arg = arg->next)
      map_nontype(q, &arg);
    rtn = f->return_type;
    map_nontype(q, &rtn);
    return;
  }

  case MLD_POINTER_TO_MEMBER_TYPE: {
    struct MLD_pointer_to_member* pmr = ut->the_type.ptr_to_member;
    if (!(pmr->type_modified->ut) || 
	pmr->type_modified->ut->type_of_type != MLD_NONTYPE_TYPE) {
      map_nontype_ut(q, pmr->type_modified->ut);
      return;
    }
    ut = pmr->type_modified->ut;
    rt = pmr->type_modified;
    break;
  }

  case MLD_CONSTANT_TYPE:
  case MLD_ADDRESS_TYPE:
    return;

  case MLD_NONTYPE_TYPE:
    break;

  default:
    return;
  }

  if (!rt) return;

  nontype_str = ut->the_type.nontype->nontype_name;
  status = _number(nontype_str, 1, &end_of_number, &id);
  if (status != MLD_SOMETHING_RECOGNIZED)
    return;
  i = end_of_number + 1;
  level = 1;
  if (strstr(nontype_str, "_") && nontype_str[i] == '_')
    status = _number(nontype_str, i+1, &end_of_number, &level);
  if (status != MLD_SOMETHING_RECOGNIZED)
    return;
  if (level > 1)
    q = get_mz_qual(q, level);

  arg = q->q->name.template_name->args;
  for (i = 1; i < id && arg; i++)
    arg = arg->next;
  if (i < id || !arg)
    return;
  _delete_unmodified_type(rt->ut);
  if (rt->shared_ut)
    printf("Please report a demangle internal error <shared_ut unexpected TRUE>\n");
  rt->shared_ut = TRUE;
  _set_shared_id(arg->ut);
  rt->ut = arg->ut;

} /* map_nontype_ut */

static
struct MLD_qualification* get_mz_qual(/* Return qualification */
	 struct MLD_qualification* q, /* IN: May contain template args. */
	 int level)		      /* IN: level of qualification */
{
  struct MLD_qualification* rq;
  struct MLD_type* arg;
  int i = 1;

  for (rq = q; rq && i <= level; rq = rq->next, i++) {
    if (!(rq->q->mangled_template_name))
      continue;
    if (i == level)
      return rq;
  }  /* for rq ... */
  return q;
}

void _cleanup_onerror (struct MLD_type_modifiers* ftml,
		       struct MLD_qualification* q,
		       struct MLD_type* a,
		       struct MLD_type* rtn_type)
{
  if (ftml)
    DEMANGLE_FREE(ftml);
  if (q)
    _delete_qualifications(q);
  if (a)
    _delete_type_list(a);
  if (rtn_type)
    _delete_type_structure(rtn_type);
  return;
}
