#pragma required_pointer_size long
#ifndef lint
static const char sccsid[] = "@(#)mangled_variable_name.c	1.1     2/10/94";
#endif
/*
*****************************************************************************
**                                                                          *
**                         COPYRIGHT (c) 1991, 1996 BY                      *
**             DIGITAL EQUIPMENT CORPORATION, MAYNARD, MASS.                *
**                          ALL RIGHTS RESERVED                             *
**                                                                          *
**  THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND  COPIED  *
**  ONLY  IN  ACCORDANCE  WITH  THE  TERMS  OF  SUCH  LICENSE AND WITH THE  *
**  INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR  ANY  OTHER  *
**  COPIES  THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY  *
**  OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF  THE  SOFTWARE  IS  HEREBY  *
**  TRANSFERRED.                                                            *
**                                                                          *
**  THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE  WITHOUT  NOTICE  *
**  AND  SHOULD  NOT  BE  CONSTRUED  AS  A COMMITMENT BY DIGITAL EQUIPMENT  *
**  CORPORATION.                                                            *
**                                                                          *
**  DIGITAL ASSUMES NO RESPONSIBILITY FOR THE USE OR  RELIABILITY  OF  ITS  *
**  SOFTWARE ON EQUIPMENT WHICH IS NOT SUPPLIED BY DIGITAL.                 *
**                                                                          *
*****************************************************************************
**++
**  PRODUCT:
***
**	DEC C++ Demangler
**
**  FILE:  
**
**      mangled_variable_name.c
**
**  ABSTRACT:
**
**	Grammar element that parses a mangled_name.
**
**  ENVIRONMENT:
**
**	User Mode
**
**  AUTHORS:
**
** 	Shell S. Simpson (DECwest Engineering)
**
**  MODIFICATION HISTORY:
**
**	 8-Jul-1996	Harold Seigel
**      Check lengths before calling strcpy to prevent overwriting the stack.
**
**	30-Dec-1997	Shi-Jung Kao
**	Process unnamed member variables.
**
**	24-Mar-1997	Shi-Jung Kao
**	Remove a day1 bug, of use an uninitialized variable.
**
**	03-Apr-1998	Shi-Jung Kao
**	_delete_qualifications(q) to free all mallocs.
**
**	01-Mar-2000	Shi-Jung Kao
**	Added DEMANGLE_TRACEIN/TRACEOUT.
**--
**/


#ifdef CXX_NT
#include "defs.h"
#include "mn_nm.h"
#include "gram_elm.h"
#include "util.h"
#else
#include "demangle.h"
#include "grammar_elements.h"
#endif

#include "demangle_internal.h"

#include <stdlib.h>
#include <stddef.h>
#include <stdio.h>
#include <string.h>

/* language element particular definitions */
#define ASCII_ZERO 48

int /* recognition status */
_mangled_variable_name
	(const char* s,		/* IN:  string to be parsed */
     int start,			/* IN:  index of first unparsed character */
     int* end,			/* OUT: index of last parsed character */
     struct MLD_mangled_variable_name** mvn,
				/* OUT: struct containing mangled var name */
     char* str_before_name,	/* OUT: string before mangled var name */
     char* str_replaced_by_name,/* OUT: string replaced by mangled var name */
     char* str_after_name,	/* OUT: string after mangled var name */
     int size_of_string)	/* IN:	max size of the strings listed above */
{
  /* variables common to all language elements */
  int ci;  /* current index */
  char ch; /* current character */
  
  /* variables particular to this language element */
  int status;
  int end_of_grammar_component;
  struct MLD_mangled_variable_name* temp_mn;
  char bs[STRSIZ];		/* string before demangled string */
  char rs[STRSIZ];		/* string replaced by demangled string */
  char as[STRSIZ];		/* string after demangled string */
  char id[STRSIZ];		/* identifier string */
  struct MLD_qualification* q=0;/* qualification */
  char class[STRSIZ];		/* class string */
  int end_of_mn;		/* index of last character in s of mn */

  int num_du = 0;		/* number of double underscores */
  int du_count;                 /* looping variable that determines the
                                   underscore number that will be used
                                   in the identifier */
  
  DEMANGLE_TRACEIN(_mangled_variable_name);

  num_du = _num_dbl_uscores(s, start);

  for (du_count=1; du_count<=num_du; du_count++)
  { /* start of loop that repeatled check for double underscores */
  
  
  /* initialization particular to language element */
  temp_mn = 0L;
  *mvn = 0L;
  
  /*
  **
  ** Get the identifier
  **
  */

  ch = s[ci = start]; /* initialize ci and ch */
  status = _identifier(s, ci, &end_of_grammar_component, du_count,
                                           id, STRSIZ, bs, STRSIZ);
  switch (status)
  {    
    case MLD_NOTHING_RECOGNIZED:
      continue; /* try to recognize again */
      break;

    case MLD_SOMETHING_RECOGNIZED:
      /* Process the unnamed variable */
      if (strncmp(id, UNNAMED_MEMBER_VARIABLE_TOKEN, UNNAMED_TOKEN_LEN) == 0 &&
	  id[3] >= '1' && id[3] <= '9') {
	/* Note, in __Vnnn format, nnn could not have a leading zero. */
	strcpy(id, UNNAMED_MEMBER_VARIABLE_STRING);
      }
      break;
    
    case MLD_ERROR:
      /* if error occured, return ERROR */
      *end = start - 1; /* nothing recognized */
      *mvn = 0L; /* no mangled name */
      
      if (size_of_string > 0) /* copy if enough room */
      {
        strcpy(str_before_name,"");
        strcpy(str_replaced_by_name,"");
      }
      
      if ((strlen(s) - start) < size_of_string) /* copy if enough room */
      {
        strcpy(str_after_name, &s[start]); /* everything is "after" */
      }
      
      DEMANGLE_TRACEOUT(_mangled_variable_name);
      return MLD_ERROR;
      break;
  }
  ch = s[ci = (end_of_grammar_component+1)]; /* update ci and ch */


  /*
  **
  ** Get the "__"
  **
  */

  if (!((s[ci] == '_') && (s[ci+1] == '_')))
  {
    continue;
  }
  ch = s[ci = (ci+2)]; /* update ci and ch */
  if (strncmp (&s[ci], "tm__", 4) == 0) {
    /* Advance du_count accordingly if is "__tm__" case. */
    du_count++;
  }


  /*
  ** deallocate memory allocated by a possible previous call to
  ** _parse_qualifications().
  */
  _delete_qualifications(q);
  q=0;

  /*
  **
  ** Get the qualifications
  **
  */

  status = _parse_qualifications(s, ci, &end_of_grammar_component, &q);
 
  switch (status)
  {    
    case MLD_NOTHING_RECOGNIZED:
      continue;
    break;
    
    case MLD_ERROR:
      /* if error occured, return ERROR */
      *end = start - 1; /* nothing recognized */
      *mvn = 0L; /* no mangled name */
      
      if (size_of_string > 0) /* copy if enough room */
      {
        strcpy(str_before_name,"");
        strcpy(str_replaced_by_name,"");
      }
      
      if ((strlen(s) - start) < size_of_string) /* copy if enough room */
      {
        strcpy(str_after_name, &s[start]); /* everything is "after" */
      }
      
      DEMANGLE_TRACEOUT(_mangled_variable_name);
      return MLD_ERROR;
    break;
  }
  ch = s[ci = (end_of_grammar_component+1)]; /* update ci and ch */


  /*
  **
  ** Check if there is an external suffix, if there is discard it
  **
  */

  status = _external_suffix(s, end_of_grammar_component+1, 
			    &end_of_grammar_component);

  switch (status)
  {
    case MLD_NOTHING_RECOGNIZED: /* OK not to recognize */
    break;

    case MLD_ERROR:
      /* if error occured, return ERROR */
      *end = start - 1; /* nothing recognized */
      *mvn = 0L; /* no mangled name */

      if (size_of_string > 0) /* copy if enough room */
      {
        strcpy(str_before_name,"");
        strcpy(str_replaced_by_name,"");
      }

      if ((strlen(s) - start) < size_of_string) /* copy if enough room */
      {
        strcpy(str_after_name, &s[start]); /* everything is "after" */
      }

      DEMANGLE_TRACEOUT(_mangled_variable_name);
      return MLD_ERROR;
    break;
  }
  ch = s[ci = (end_of_grammar_component+1)]; /* update ci and ch */
 

  /*
  **
  ** Check for white space
  **
  */
  
  end_of_mn = end_of_grammar_component;
  status = _white_space(s, end_of_grammar_component + 1, &end_of_grammar_component);
 
  switch (status)
  {    
    case MLD_NOTHING_RECOGNIZED:
      continue;
    break;
    
    case MLD_ERROR:
      /* if error occured, return ERROR */
      
      *end = start - 1; /* nothing recognized */
      *mvn = 0L; /* no mangled name */
      
      if (size_of_string > 0) /* copy if enough room */
      {
        strcpy(str_before_name,"");
        strcpy(str_replaced_by_name,"");
      }
      
      if ((strlen(s) - start) < size_of_string) /* copy if enough room */
      {
        strcpy(str_after_name, &s[start]); /* everything is "after" */
      }
      
      DEMANGLE_TRACEOUT(_mangled_variable_name);
      return MLD_ERROR;
    break;
  }
 
  /*
  **
  ** All the trials have been passed.  Now it's time to create the
  ** mangled variable name structure and set it appropriately.
  **
  */
 
  temp_mn = (struct MLD_mangled_variable_name*) malloc(sizeof(struct MLD_mangled_variable_name));
  DEMANGLE_RECORD_ALLOC(temp_mn);
  if (temp_mn == 0L) {
    DEMANGLE_TRACEOUT(_mangled_variable_name);
    return MLD_ERROR; /* error if no memory allocated */
  }

  temp_mn->identifier = (char*) malloc(strlen(id)+1);
  DEMANGLE_RECORD_ALLOC(temp_mn->identifier);
  if (temp_mn->identifier == 0L)
  {
    DEMANGLE_FREE(temp_mn);
    DEMANGLE_TRACEOUT(_mangled_variable_name);
    return MLD_ERROR; /* error if no memory allocated */
  }
  
  strcpy(temp_mn->identifier, id);
  
  temp_mn->qualifications = q;


      
  if (strlen(bs) < size_of_string) /* copy if enough room */
  {
    strcpy(str_before_name, bs);
  }
      
  if ((end_of_mn - (strlen(bs) + start) + 1) < size_of_string) /* copy if enough room */
  {
    strncpy(str_replaced_by_name,
            &s[start + strlen(bs)],
            end_of_mn - (strlen(bs) + start) + 1 );
    str_replaced_by_name[(end_of_mn - (strlen(bs) + start) + 1)] = 0; /* null terminate */
  }
      
  if (strlen(&s[end_of_mn+1]) < size_of_string) /* copy if enough room */
  {
    strcpy(str_after_name, &s[end_of_mn+1]);
  }
      
  *mvn = temp_mn;   
  *end = end_of_mn; /* end of mangled name */
  DEMANGLE_TRACEOUT(_mangled_variable_name);
  return MLD_SOMETHING_RECOGNIZED;
  
  } /* end of loop that repeatedly checks for double underscores */
  
  
  /* if no identifier was recognized, return NOTHING_RECOGNIZED */
  *end = start - 1; /* nothing recognized */
  *mvn = 0L; /* no mangled name */
  _delete_qualifications(q);

  if ((strlen(s) - start) < size_of_string) /* copy if enough room */
  {
    strcpy(str_before_name, &s[start]); 
  }
      
  if (size_of_string > 0) /* copy if enough room */
  {
    strcpy(str_replaced_by_name,"");
    strcpy(str_after_name,"");
  }

  DEMANGLE_TRACEOUT(_mangled_variable_name);
  return MLD_NOTHING_RECOGNIZED; /* nothing has been recognized */
}
