#pragma required_pointer_size long
#ifndef lint
static const char sccsid[] = "@(#)pascal_string.c	1.1     2/10/94";
#endif
/*
*****************************************************************************
**                                                                          *
**                         COPYRIGHT (c) 1991 BY                            *
**             DIGITAL EQUIPMENT CORPORATION, MAYNARD, MASS.                *
**                          ALL RIGHTS RESERVED                             *
**                                                                          *
**  THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND  COPIED  *
**  ONLY  IN  ACCORDANCE  WITH  THE  TERMS  OF  SUCH  LICENSE AND WITH THE  *
**  INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR  ANY  OTHER  *
**  COPIES  THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY  *
**  OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF  THE  SOFTWARE  IS  HEREBY  *
**  TRANSFERRED.                                                            *
**                                                                          *
**  THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE  WITHOUT  NOTICE  *
**  AND  SHOULD  NOT  BE  CONSTRUED  AS  A COMMITMENT BY DIGITAL EQUIPMENT  *
**  CORPORATION.                                                            *
**                                                                          *
**  DIGITAL ASSUMES NO RESPONSIBILITY FOR THE USE OR  RELIABILITY  OF  ITS  *
**  SOFTWARE ON EQUIPMENT WHICH IS NOT SUPPLIED BY DIGITAL.                 *
**                                                                          *
*****************************************************************************
**++
**  PRODUCT:
***
**	DEC C++ Demangler
**
**  FILE:  
**
**      pascal_string.c
**
**  ABSTRACT:
**
**	Grammar element that parses a pascal string.
**
**  ENVIRONMENT:
**
**	User Mode
**
**  AUTHORS:
**
** 	Shell S. Simpson (DECwest Engineering)
**
**  MODIFICATION HISTORY:
**
** 05-Jan-1995  Harold Seigel 	Add support for demangling names truncated
**				to 1022 characters.
**--
**/


#ifdef CXX_NT
#include "defs.h"
#include "mn_nm.h"
#include "gram_elm.h"
#include "util.h"
#else
#include "demangle.h"
#include "grammar_elements.h"
#endif

#include <stdlib.h>
#include <stddef.h>
#include <stdio.h>
#include <string.h>

/* language element particular definitions */
#define SPACE 32
#define TAB 9
#define CARRIAGE_RETURN 13
#define LINE_FEED 10
#define STRING_TERMINATOR 0

int /* recognition status */
_pascal_string(const char* s,		/* IN:  string to be parsed */
              int start,	/* IN:  index of first unparsed character */
              int* end,		/* OUT: index of last parsed character */
              char* string,	/* OUT: integer value of number recognized */
              int string_size)	/* IN:  maximum possible length of string */
{
  /* variables common to all language elements */
  int ci;  /* current index */
  char ch; /* current character */
  
  /* variables particular to this language element */
  int status;
  int end_of_number;
  int number_of_chars_in_string;
  int number_of_result_chars_in_string;
  int start_of_string;
  int i;
  int str_len;
  int index_of_found_local_class_with_level = 0;
  /*
   * If the string is not a truncated string or if the starting point is not
   * within the truncated part of the string, then call _number() to parse
   * it to determine the number of characters in the string
   *
   */
  if (!treat_as_trunc || (start < TRUNC_LENGTH) || (start >= MAX_NAME_LENGTH)) {

      status = _number(s, start, &end_of_number, &number_of_chars_in_string);

      /*
       * Take care of case where string is truncated but the length has
       * not been changed and so extends beyond the end of the string.
       */
      if (treat_as_trunc && (status == MLD_SOMETHING_RECOGNIZED)) {
	  /*
	   * Adjust the value returned by _number() if it is too long.
	   * We could be looking at just a piece of a trunctated string
	   * so compare the returned length with the actual length, not
	   * with MAX_NAME_LENGTH.
	   */
	  str_len = strlen(s+end_of_number);
	  if (number_of_chars_in_string >= str_len) {
              number_of_chars_in_string = str_len - 1;
	  }
      }

  } else {
      /*
       * Treat the string as a truncated string.  Do not call _number() because
       * its result cannot be trusted.  Instead, pretend it passed and set
       * the proper variables appropriately.
       */
      number_of_chars_in_string = MAX_NAME_LENGTH - start;
      status = MLD_SOMETHING_RECOGNIZED;
      end_of_number = start-1;
  }


  if (status == MLD_NOTHING_RECOGNIZED)
  {
    *end = start - 1;
    if (string_size > 0) /* make string null if it's large enough */
    {
      strcpy(string, "");
      return MLD_NOTHING_RECOGNIZED; 
    }
    else
    {
      return MLD_ERROR;
    }
  }
  
  start_of_string = end_of_number + 1;
  

  /*
  ***
  *** check to see if the string is valid 
  ***
  */

  /* initialization particular to language element */


  ch = s[ci = (end_of_number + 1)]; /* initialize ch and ci */
  for (i = 0 ; i < number_of_chars_in_string ; i++)
  {
    if ( (ch == SPACE)             || /* is it a space */
         (ch == LINE_FEED)         || /* is it a line feed */
         (ch == TAB)               || /* is it a tab */
         (ch == STRING_TERMINATOR) || /* is it a string terminator */
         (ch == CARRIAGE_RETURN) )    /* is it a return */
    {
      /* string has ended abruptly */
      *end = start -1;  
      if (string_size > 0) /* make string null if it's large enough */
      {
        strcpy(string, ""); 
        return MLD_NOTHING_RECOGNIZED;
      }
      else
      {
        return MLD_ERROR;
      }
    }
    if (ch == '_' && i+2 < number_of_chars_in_string &&
	s[ci+1] == '_' && s[ci+2] == LEVEL_TOKEN)
      index_of_found_local_class_with_level = i;
    ch = s[++ci]; /* increment ch and ci */
  }
  
  /*
  ***
  *** the string has passed all the trials, package it and send it home 
  ***
  */

  if (index_of_found_local_class_with_level > 0)
    number_of_result_chars_in_string = index_of_found_local_class_with_level;
  else
    number_of_result_chars_in_string = number_of_chars_in_string;
  if (number_of_result_chars_in_string < string_size)
  {
    /* copy characters from s into string */
    strncpy(string, &s[start_of_string], number_of_result_chars_in_string);
    string[number_of_result_chars_in_string] = 0;
    *end = start_of_string + number_of_chars_in_string - 1; 
    return MLD_SOMETHING_RECOGNIZED;
  }
  else
  {
    *end = start - 1;
    return MLD_ERROR;
  }
}
