#pragma required_pointer_size long
#ifndef lint
static const char sccsid[] = "@(#)qualifications.c	1.1     2/10/94";
#endif
/*
*****************************************************************************
**                                                                          *
**                         COPYRIGHT (c) 1991, 1996 BY                      *
**             DIGITAL EQUIPMENT CORPORATION, MAYNARD, MASS.                *
**                          ALL RIGHTS RESERVED                             *
**                                                                          *
**  THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND  COPIED  *
**  ONLY  IN  ACCORDANCE  WITH  THE  TERMS  OF  SUCH  LICENSE AND WITH THE  *
**  INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR  ANY  OTHER  *
**  COPIES  THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY  *
**  OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF  THE  SOFTWARE  IS  HEREBY  *
**  TRANSFERRED.                                                            *
**                                                                          *
**  THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE  WITHOUT  NOTICE  *
**  AND  SHOULD  NOT  BE  CONSTRUED  AS  A COMMITMENT BY DIGITAL EQUIPMENT  *
**  CORPORATION.                                                            *
**                                                                          *
**  DIGITAL ASSUMES NO RESPONSIBILITY FOR THE USE OR  RELIABILITY  OF  ITS  *
**  SOFTWARE ON EQUIPMENT WHICH IS NOT SUPPLIED BY DIGITAL.                 *
**                                                                          *
*****************************************************************************
**++
**  PRODUCT:
***
**	DEC C++ Demangler
**
**  FILE:  
**
**      qualifications.c
**
**  ABSTRACT:
**
**	Grammar element that parses type modifiers.
**
**  ENVIRONMENT:
**
**	User Mode
**
**  AUTHORS:
**
** 	Shell S. Simpson (DECwest Engineering)
**
**  MODIFICATION HISTORY:
**
**	05-Jan-1995  Harold Seigel
**	Before calling _complex_type(), checked if at the end of a 
**	truncated name.
**
**	19-Feb-1998  Shi-Jung Kao
**	_delete_qualifications before return ERROR/NOTHING_RECOGNIZED.
**
**	27-Oct-1999  Shi-Jung Kao
**	Added way of recovering from "assuming ansi".
**
**	04-Nov-1999  Shi-Jung Kao
**	Limited the class designation to "template__qualifiers".
**
**	15-Dec-1999  Shi-Jung Kao
**	Separated qualification level between ANSI and nonANSI.
**
**	07-Feb-2000  Shi-Jung Kao
**	Accepted a SPECIALized function. e.g. f__tm__2_v__S1CFv_v
**	Added DEMANGLE_TRACEIN/TRACEOUT.
**--
**/


#ifdef CXX_NT
#include "defs.h"
#include "mn_nm.h"
#include "gram_elm.h"
#include "util.h"
#else
#include "demangle.h"
#include "grammar_elements.h"
#endif

#include "demangle_internal.h"

#include <stdlib.h>
#include <stddef.h>
#include <stdio.h>
#include <string.h>

/* language element particular definitions */
#define ASCII_ZERO 48

int /* recognition status */
_parse_qualifications
	(const char* s,		/* IN:  string to be parsed */
	 int start,		/* IN:  index of first unparsed character */
	 int* end,		/* OUT: index of last parsed character */
	 struct MLD_qualification** ql)	/* OUT: qualification list (allocates memory) */
{
  /* variables common to all language elements */
  int ci;  /* current index */
  char ch; /* current character */
  
  /* variables particular to this language element */
  int status = MLD_NOTHING_RECOGNIZED;
  int end_of_complex;
  struct MLD_complex* q;
  struct MLD_qualification* temp_ql;
  struct MLD_qualification* current_ql = 0L;
  struct MLD_qualification* cql;
  int number_of_qualifications, i;
  int number, end_of_number;
  int ansi_form = FALSE;
  char* temp_ansi_form = start>2 ? strstr(&s[start-2], "__tm__") : 0;

  /*
  ***
  *** Check to see if there are qualifications.
  *** NOTE:  The class name is a qualification, but it's handled separately.  
  ***
  */
  
  DEMANGLE_TRACEIN(_parse_qualifications);

  /* default values */
  *end = start - 1; 
  *ql = 0L; 
  
  ch = s[ci = start]; /* initialize ch and ci */
  if(ch == QUALIFIED_TOKEN)
  {  
     char ch_nonansi;
     int number_of_qualifications_nonansi,ci_nonansi;

     /* handle the case of having qualifications */
     ch = s[++ci]; /* increment ch and ci */
     
     /* get the number of qualifications (in the form of a single digit) */
     /* Unless its a '?' i.e. more levens than 9 or under ansi */
     if (ch == '?') {
       number_of_qualifications = 9999;
       ch = s[++ci]; /* increment ch and ci */
     } else {
       /* Or this is a ansi form, i.e. Q12_ with a level of 12 */

       status = _number(s, ci, &end_of_number, &number);
       if (status != MLD_SOMETHING_RECOGNIZED) return status;
       if (end_of_number+2 > strlen(s)) return MLD_NOTHING_RECOGNIZED;

       if (s[end_of_number+1] == '_' && s[end_of_number+2] != '_') {
	 /* Exclude from case of: f__XRCQ115__N1AIykOuvNyIN1C */

	 /* Save for backing-up if is nonansi after all. 
	  * Case: cxxc_bugs7125.4:
	  *	digitize__Q18_RWrwstd17digit_writer_baseXi
	  */
	 ci_nonansi = ci+1;
	 number_of_qualifications_nonansi = ch - ASCII_ZERO;
	 ch_nonansi = s[ci_nonansi];

	 /* Assume its ansi for now. */
	 ansi_form = TRUE;
	 number_of_qualifications = number - 1;
	 ci = end_of_number+2;
	 ch = s[ci];
       } else if ((ch >= '0') && (ch <= '9')) {
	 number_of_qualifications = ch - ASCII_ZERO;
	 ch = s[++ci]; /* increment ch and ci */
       }
     }

     
     /* get first qualifier */
     *ql = 0L; /* initialize *ql to be empty */    
     status = _complex_type(s, ci, &end_of_complex, &q);
     if (status != MLD_SOMETHING_RECOGNIZED) {
       ci = ci_nonansi;
       status = _complex_type(s, ci, &end_of_complex, &q);
       if (status != MLD_SOMETHING_RECOGNIZED)
	 return status;
       ansi_form = FALSE;
       number_of_qualifications = number_of_qualifications_nonansi;
     }
     ch = s[ci = (end_of_complex+1)]; /* update ch and ci */
     
     /* create and instantiate qualifier link */
     temp_ql = (struct MLD_qualification*) malloc(sizeof(struct MLD_qualification));
     DEMANGLE_RECORD_ALLOC(temp_ql);
     if (temp_ql == 0L) return MLD_ERROR; /* error, no memory allocated */
     
     temp_ql->q = q; /* set the qualification equal to the complex type just 
                        found */
     temp_ql->next = 0L; /* no next pointer */
     *ql = temp_ql;
     current_ql = temp_ql;
           
     for (i = 1 ; i < number_of_qualifications ; i++) /* get the 2nd and beyond */
     {  
       status = _complex_type(s, ci, &end_of_complex, &q);
       if (status != MLD_SOMETHING_RECOGNIZED)
       {
	 if (number_of_qualifications == 9999 && !ansi_form)
	   break;

         _delete_qualifications(*ql);
	 *ql = 0L; 
	 DEMANGLE_TRACEOUT(_parse_qualifications);
         return status;
       }
       ch = s[ci = (end_of_complex+1)]; /* update ch and ci */
        
       /* create and instantiate type modifer link */
       temp_ql = (struct MLD_qualification*) malloc(sizeof(struct MLD_qualification));
       DEMANGLE_RECORD_ALLOC(temp_ql);
       if (temp_ql == 0L) {
	 _delete_complex(q);
         _delete_qualifications(*ql);
	 *ql = 0L; 
	 DEMANGLE_TRACEOUT(_parse_qualifications);
	 return MLD_ERROR; /* error, no memory allocated */
       }
     
       temp_ql->q = q; /* set the qualification equal to the complex type just 
                          found */

       temp_ql->next = 0L; /* no next pointer */
       
       current_ql->next = temp_ql;
       current_ql = temp_ql;
     }
  }


  /* check for a class designation */
  if ((number_of_qualifications != 9999 || ansi_form) &&
      ((!treat_as_trunc) || (ci < MAX_NAME_LENGTH))) {
      int cj, end_of_class;
      struct MLD_complex* cq;

      q = 0L;
      status = _complex_type(s, ci, &end_of_complex, &q);
      if (status != MLD_SOMETHING_RECOGNIZED)
      {
	_delete_complex(q);
        _delete_qualifications(*ql);
	*ql = 0L; 
	DEMANGLE_TRACEOUT(_parse_qualifications);
        return status;
      }

      /* foo__tm__2_c__Q1_1C1DFv_v
       * foo__tm__2_c__1CFv_v
       *    ^^^^^^^^^^^ <======= previous one, a template.
       * foo__tm__2_c__S1CFv_v
       *               ^ <======  and Specialized
       *
       * Now, we are really looking at the qualifiers.
       */
      if (temp_ansi_form && q->mangled_template_name &&
	  s[end_of_complex-1] == '_' && s[end_of_complex] == '_') {
	/* Get the class designation, the previous q might be a template. */
	cj = end_of_complex+1;
	if (s[cj] == SPECIAL_TOKEN)
	  cj++;
	if (s[cj] == QUALIFIED_TOKEN) {
	  status = _parse_qualifications(s, cj, &end_of_class, &temp_ql);
	  if (status != MLD_SOMETHING_RECOGNIZED) {
	    _delete_complex(q);
	    _delete_qualifications(*ql);
	    *ql = 0L; 
	    DEMANGLE_TRACEOUT(_parse_qualifications);
	    return status;
	  }
	} else {
	  status = _complex_type(s, cj, &end_of_class, &cq);
	  if (status == MLD_SOMETHING_RECOGNIZED) {
	    temp_ql = (struct MLD_qualification*) 
				malloc(sizeof(struct MLD_qualification));
	    if (temp_ql == 0L) {
	      _delete_complex(q);
	      _delete_complex(cq);
	      _delete_qualifications(*ql);
	      *ql = 0L; 
	      DEMANGLE_TRACEOUT(_parse_qualifications);
	      return MLD_ERROR; /* error, no memory allocated */
	    }
	    DEMANGLE_RECORD_ALLOC(temp_ql);
	    temp_ql->q = cq;
	    temp_ql->next = 0L;
	  }
	}
	if (status == MLD_SOMETHING_RECOGNIZED) {
	  struct MLD_qualification* next_ql;  
	  end_of_complex = end_of_class;
	  if (*ql == 0L) {
	    *ql = temp_ql;
	  } else {
	    current_ql->next = temp_ql;
	  }
	  current_ql = temp_ql;
	  /* Update current_ql if temp_ql has a full lists. */
	  for (next_ql = temp_ql->next; next_ql; next_ql = next_ql->next)
	    current_ql = next_ql;
	}
      }
      
      /* create and instantiate qualifier link */
      temp_ql = (struct MLD_qualification*) 
				malloc(sizeof(struct MLD_qualification));
      DEMANGLE_RECORD_ALLOC(temp_ql);
      if (temp_ql == 0L) {
	_delete_complex(q);
	_delete_qualifications(*ql);
	*ql = 0L; 
	DEMANGLE_TRACEOUT(_parse_qualifications);
	return MLD_ERROR; /* error, no memory allocated */
      }
     
      temp_ql->q = q; /* set the qualification equal to the complex type just 
                        found */
      temp_ql->next = 0L; /* no next pointer */     
  
      if (*ql == 0L)
      {
        *ql = temp_ql;
      }
      else
      {
        current_ql->next = temp_ql;
      }
  }
  
  DEMANGLE_TRACEOUT(_parse_qualifications);
  if (*ql == 0L) { /* Nothing Recognized */
    *end = start - 1;
    return MLD_NOTHING_RECOGNIZED;
  } else { /* Something Recognized */
    *end = end_of_complex;
    return MLD_SOMETHING_RECOGNIZED;
  }
}
