#pragma required_pointer_size long
#ifndef lint
static const char sccsid[] = "@(#)recover_from_error.c	1.1     2/10/94";
#endif
/*
*****************************************************************************
**                                                                          *
**                         COPYRIGHT (c) 1991 BY                            *
**             DIGITAL EQUIPMENT CORPORATION, MAYNARD, MASS.                *
**                          ALL RIGHTS RESERVED                             *
**                                                                          *
**  THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND  COPIED  *
**  ONLY  IN  ACCORDANCE  WITH  THE  TERMS  OF  SUCH  LICENSE AND WITH THE  *
**  INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR  ANY  OTHER  *
**  COPIES  THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY  *
**  OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF  THE  SOFTWARE  IS  HEREBY  *
**  TRANSFERRED.                                                            *
**                                                                          *
**  THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE  WITHOUT  NOTICE  *
**  AND  SHOULD  NOT  BE  CONSTRUED  AS  A COMMITMENT BY DIGITAL EQUIPMENT  *
**  CORPORATION.                                                            *
**                                                                          *
**  DIGITAL ASSUMES NO RESPONSIBILITY FOR THE USE OR  RELIABILITY  OF  ITS  *
**  SOFTWARE ON EQUIPMENT WHICH IS NOT SUPPLIED BY DIGITAL.                 *
**                                                                          *
*****************************************************************************
**++
**  PRODUCT:
***
**	DEC C++ Demangler
**
**  FILE:  
**
**      recover_from_error.c
**
**  ABSTRACT:
**
**	This function gets called when an error occurs.  It decides where to continue
**  scanning.
**
**  ENVIRONMENT:
**
**	User Mode
**
**  AUTHORS:
**
** 	Shell S. Simpson (DECwest Engineering)
**
**  MODIFICATION HISTORY:
**
**      $Id: 
**      $Log: 
**
**--
**/


#ifdef CXX_NT
#include "defs.h"
#include "mn_nm.h"
#include "gram_elm.h"
#include "util.h"
#else
#include "demangle.h"
#include "grammar_elements.h"
#endif

#include <stdlib.h>
#include <stddef.h>
#include <stdio.h>
#include <string.h>

int /* recognition status */
_recover_from_error(const char* s,	/* IN:  string to be parsed */
                  int start,		/* IN:  index of first unparsed character */
                  int* end,			/* OUT: index of last parsed character */
                  char* bs,			/* OUT: string that gets passed over */
                  int bs_max)		/* IN:	maximum size of bs */
{
  int ch;
  int ci;
  int end_of_bs;
  int size_of_bs;

  ch = s[ci = start]; /* initialize ch and ci */
  /* check if starting character is null */
  if (ch == 0)
  {
    *end = start - 1; /* we don't "recognize" null */
    if (bs_max > 0) strcpy(bs,"");
    return MLD_NOTHING_RECOGNIZED; /* we're at the end of the string! */
  }
  
  /* eat initial white space, if any */
   while( (ch == 32)       || /* is it a space */
            (ch == 10)     || /* is it a line feed */
            (ch == 9)      || /* is it a tab */
            (ch == 13) )      /* is it a return */
  {
     ch = s[++ci]; /* increment ch and ci */
  }

  /* wait for white space */ 
  while( !( (ch == 32)     || /* is it a space */
            (ch == 10)     || /* is it a line feed */
            (ch == 9)      || /* is it a tab */
            (ch == 13) ) )    /* is it a return */
  {
     if (ch == 0)
     {
       *end = start - 1; /* we don't recognize null */
       end_of_bs = ci - 1;
       size_of_bs = end_of_bs - start + 1;
       if (size_of_bs < bs_max)
       {
         strncpy(bs, &s[start], size_of_bs);
         bs[size_of_bs] = 0; /* null terminate */
       }
       else
       {
         return MLD_ERROR;
       }
       return MLD_NOTHING_RECOGNIZED; /* no new string to parse */
     }
     ch = s[++ci]; /* increment ch and ci */
  }
  
  /* white space recognized, end location on */
  end_of_bs = ci - 1; /* last character of bs is one less than the ws */
  *end = ci - 1;
  size_of_bs = end_of_bs - start + 1;
  if (size_of_bs < bs_max)
  {
    strncpy(bs, &s[start], size_of_bs);
    bs[size_of_bs] = 0; /* null terminate */
  }
  else
  {
    *end = start - 1;
    return MLD_ERROR;
  }
  return MLD_SOMETHING_RECOGNIZED; /* no new string to parse */
}
