#pragma required_pointer_size long
#ifndef lint
static const char sccsid[] = "@(#)template_arguments.c	1.1     2/10/94";
#endif
/*
*****************************************************************************
**                                                                          *
**                         COPYRIGHT (c) 1991 BY                            *
**             DIGITAL EQUIPMENT CORPORATION, MAYNARD, MASS.                *
**                          ALL RIGHTS RESERVED                             *
**                                                                          *
**  THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND  COPIED  *
**  ONLY  IN  ACCORDANCE  WITH  THE  TERMS  OF  SUCH  LICENSE AND WITH THE  *
**  INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR  ANY  OTHER  *
**  COPIES  THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY  *
**  OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF  THE  SOFTWARE  IS  HEREBY  *
**  TRANSFERRED.                                                            *
**                                                                          *
**  THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE  WITHOUT  NOTICE  *
**  AND  SHOULD  NOT  BE  CONSTRUED  AS  A COMMITMENT BY DIGITAL EQUIPMENT  *
**  CORPORATION.                                                            *
**                                                                          *
**  DIGITAL ASSUMES NO RESPONSIBILITY FOR THE USE OR  RELIABILITY  OF  ITS  *
**  SOFTWARE ON EQUIPMENT WHICH IS NOT SUPPLIED BY DIGITAL.                 *
**                                                                          *
*****************************************************************************
**++
**  PRODUCT:
***
**	DEC C++ Demangler
**
**  FILE:  
**
**      template_arguments.c
**
**  ABSTRACT:
**
**	Grammar element that parses a list a template arguments.
**
**  ENVIRONMENT:
**
**	User Mode
**
**  AUTHORS:
**
** 	Shell S. Simpson (DECwest Engineering)
**
**  MODIFICATION HISTORY:
**
**	31-Dec-1997	Shi-Jung Kao 
**      Handle __Ti__Ti case.  It effectively has the same result as __Tii.
**
**	20-Apr-1999	Shi-Jung Kao cxxc_bugs:6619
**	Set end_of_ta_list->next to a_type only if a_type is not itself, 
**	otherwise creates a wrong loop in list.
**
**	20-Jan-2000	Shi-Jung Kao
**	Accept __ct__36D__tm__27_XCM1AFv_vLM0_L_2_n1_5f__1AFv
**      under version7.                   ^^^^^^^^^^^^^^^^^
**
**	25-Jan-2000	Shi-Jung Kao
**	Return nothing_recog if start beyond the s.
**	Added DEMANGLE_TRACEIN/TRACEOUT.
**--
**/


#ifdef CXX_NT
#include "defs.h"
#include "mn_nm.h"
#include "gram_elm.h"
#include "util.h"
#else
#include "demangle.h"
#include "grammar_elements.h"
#endif

#include "demangle_internal.h"

#include <stdlib.h>
#include <stddef.h>
#include <stdio.h>
#include <string.h>

int /* recognition status */
_template_arguments (
	     const char* s,	/* IN:  string to be parsed */
	     int start,		/* IN:  index of first unparsed character */
	     int* end,		/* OUT: index of last parsed character */
	     struct MLD_type** ta)/* OUT: template args consist of type list */
{
  /* variables common to all language elements */
  int ci = start;  /* current index */
  char ch = s[ci]; /* current character */
  
  /* variables particular to this language element */
  int status;
  int end_of_grammar_element;
  struct MLD_type* a_type;
  struct MLD_type* temp_ta;
  struct MLD_type* end_of_ta_list = 0L;
  int ansi_form = 0;
  int cprev;

#if DEC_ANSI_FORM
  /*******************************************************
     6 cases to consider:
     too__tm__3_df__XRC17Vector__tm__4_Z2Z_17Vector__tm__4_Z1Z
              ^^^^
     foo__19S__tm__10_XCPA5_i1vXv_i
                   ^^^^^^^^^^^^
     f__24C__tm__15_XCPC3IID5i__1SXv_v
                 ^^^^^^^^^^^^^^^^^
     f__tm__11_1nXCiL_1_5__XPZ1Z_v
            ^^^^^^^^^^^^^
     f3__tm__15_1n1n1nXCiL_1_5__XPZ1ZPZ2ZPZ3Z_v	  // Note, 1n1n1n 
             ^^^^^^^^^^^^^^^^^
     __ct__36D__tm__27_XCM1AFv_vLM0_L_2_n1_5f__1AFv
                    ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

   *****************************************************/
  char string[STRSIZ];
  char string2[STRSIZ];
  int end_of_pascal_string;

  *end = start - 1;
  *ta = 0L;
  if (start > strlen(s)) {
    return MLD_NOTHING_RECOGNIZED;
  }

  if (ansi_form = (s[start-1] == '_')) {
    status = _pascal_string(s, start, &end_of_pascal_string, string, STRSIZ);
    if (status == MLD_SOMETHING_RECOGNIZED) {
      for (;s[ci]!='_' && ci<start+strlen(string);ci++)
	continue;
      ch = s[++ci]; /* Skip the '_'. */
      if (ch == 'X')
	ch = s[++ci]; /* Skip the X in the 2nd case. */
    }
  }
#endif

  DEMANGLE_TRACEIN(_template_arguments);

  /* first try to get type */
  /* Limit the string to look at */
  status = _pascal_string(s, ci, &end_of_pascal_string, string, STRSIZ);
  if (ansi_form && status == MLD_SOMETHING_RECOGNIZED) {
    /* Reconstruct the string, complex type need the begining string. */
    strncpy(&string2[0], s, end_of_pascal_string+1);
    string2[end_of_pascal_string+1] = 0;
    status = _template_type(string2, ci, &end_of_grammar_element, &a_type);
  } else
    status = _template_type(s, ci, &end_of_grammar_element, &a_type);
  cprev = ci;

  *ta = 0L; /* initialize ta to point to nothing */
  while (status == MLD_SOMETHING_RECOGNIZED) {
    /* if something was recognized, then a_type points to an initialized type
       structure */
 
    int cj = end_of_grammar_element + 1;;

    /* If there is a pascal string immediately follows, then
     * replace the a_type with the nontype pascal string.  Unless
     * the "a_type" itself is a pascal string.
     * e.g. Interpret CPA5_i1v as the "address of v".
     *		      CPC3IID5i__1S as the "address of "S::i"
     *      but not   1n1n1n  into  1n1n.
     *
     * Note, because the code will skip the '_', we have to validate
     * it if is not the type of:
     *		"_1_"
     *
     * We also have to skip the "X" under ansi form.  It can have a middle
     * X within this pascal string.  i.e. f__tm__11_1nXCiL_1_5__XPZ1Z_v
     */

    /* Or under version7, if there is a LM0_L_2_n1_5f__1A immediately follows,
     * then replace the a_type with the "Pointer to member function...".
     */

    if (s[cj] == 'X') {
      end_of_grammar_element++;
      cj++;
    }

    if (ansi_form && s[cj-1] != '_' && s[cj] > '0' && s[cj] <= '9') {
      struct MLD_type* t = 0L;
      int is_pointer = _is_pointer_type(a_type);
      status = _nontype_type(s, is_pointer, cj, &end_of_grammar_element, &t);
      /* Replace the prev type, unless its a pascal string. */

      if (s[cprev] > '0' && s[cprev] <= '9') {
	/* Prev one is a pascal string. */
	/* append the type modifier to the type modifier list */
	if (*ta == 0L) { /* first type */
	  *ta = a_type;
	} else { /* subseqent type modifier */
	  if (end_of_ta_list != a_type) /* if a_type is new */
	    end_of_ta_list->next = a_type;
	}
	end_of_ta_list = a_type;
	if (t != NULL)
	  a_type = t;
      } else {
	/* cprev is not a pascal string. */
	if (status == MLD_SOMETHING_RECOGNIZED) {
	  _delete_type_structure(a_type); /* Replace the type with nontype one */
	  a_type = t;
	} else
	  _delete_type_structure(t); /* Remove the nontype one */
      }
    } else if (MLD_mangled_version == MLD_VERSION7 && s[cj-1] != '_'
	       && s[cj] == 'L' && s[cj+1] == 'M') {
      struct MLD_type* t = 0L;

      status = _template_type(s, cj, &end_of_grammar_element, &t);
      if (status == MLD_SOMETHING_RECOGNIZED) {
	_delete_type_structure(a_type); /* Replace the type with constant one */
	cj = end_of_grammar_element;
	a_type = t;
      } else
	  _delete_type_structure(t); /* Remove the nontype one */
    }

    
    /* append the type modifier to the type modifier list */
    if (*ta == 0L) { /* first type */
      *ta = a_type;
    } else { /* subseqent type modifier */
      if (end_of_ta_list != a_type) /* if a_type is new */
	end_of_ta_list->next = a_type;
    }
    end_of_ta_list = a_type;

    /*
    **
    ** Get the "__T" if necessary, we can have a case like:
    ** foo__9C__Ti__TiXv, (Demangle the same result as foo__6C__TiiXv)
    **
    */
    if (strlen(s) > cj+2 &&
	(s[cj] == '_') && (s[cj+1] == '_') && (s[cj+2] == TEMPLATE_TOKEN)) {
      end_of_grammar_element = cj + 2;
    }

    /* Limit the string to look at */
    status = _pascal_string(s, end_of_grammar_element+1, 
			    &end_of_pascal_string, string, STRSIZ);
    if (ansi_form && status == MLD_SOMETHING_RECOGNIZED) {
      /* Reconstruct the string, complex type need the begining string. */
      strncpy(&string2[0], s, end_of_pascal_string+1);
      string2[end_of_pascal_string+1] = 0;
      status = _template_type(string2, end_of_grammar_element + 1, 
                            &end_of_grammar_element, &a_type);
    } else
      status = _template_type(s, end_of_grammar_element + 1, 
                            &end_of_grammar_element, &a_type); /* allocs mem */
    cprev = cj;
  }
  
  if (status == MLD_ERROR)
  {
    /* delete existing ta */
    _delete_type_list(*ta);
    
    *ta = 0L;
    *end = start - 1;
    DEMANGLE_TRACEOUT(_template_arguments);
    return MLD_ERROR;
  }
  
  /* Handle case where nothing is recognized... */  
  if (*ta != 0L) 
  { /* Actually Something Recognized */
    *end = end_of_grammar_element;
    DEMANGLE_TRACEOUT(_template_arguments);
    return MLD_SOMETHING_RECOGNIZED;
  }
  else
  { /* Nothing Recognized */
    *ta = 0L; /* set to zero because nothing found */
    *end = start - 1;
    DEMANGLE_TRACEOUT(_template_arguments);
    return MLD_NOTHING_RECOGNIZED; /* should be at least one template argument */
  }
}
