#pragma required_pointer_size long
#ifndef lint
static const char sccsid[] = "@(#)unmodified_type.c	1.1     2/10/94";
#endif
/*
*****************************************************************************
**                                                                          *
**                         COPYRIGHT (c) 1991 BY                            *
**             DIGITAL EQUIPMENT CORPORATION, MAYNARD, MASS.                *
**                          ALL RIGHTS RESERVED                             *
**                                                                          *
**  THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND  COPIED  *
**  ONLY  IN  ACCORDANCE  WITH  THE  TERMS  OF  SUCH  LICENSE AND WITH THE  *
**  INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR  ANY  OTHER  *
**  COPIES  THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY  *
**  OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF  THE  SOFTWARE  IS  HEREBY  *
**  TRANSFERRED.                                                            *
**                                                                          *
**  THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE  WITHOUT  NOTICE  *
**  AND  SHOULD  NOT  BE  CONSTRUED  AS  A COMMITMENT BY DIGITAL EQUIPMENT  *
**  CORPORATION.                                                            *
**                                                                          *
**  DIGITAL ASSUMES NO RESPONSIBILITY FOR THE USE OR  RELIABILITY  OF  ITS  *
**  SOFTWARE ON EQUIPMENT WHICH IS NOT SUPPLIED BY DIGITAL.                 *
**                                                                          *
*****************************************************************************
**++
**  PRODUCT:
***
**	DEC C++ Demangler
**
**  FILE:  
**
**      unmodified_type.c
**
**  ABSTRACT:
**
**	Grammar element that parses an unmodified type.
**
**  ENVIRONMENT:
**
**	User Mode
**
**  AUTHORS:
**
** 	Shell S. Simpson (DECwest Engineering)
**
**  MODIFICATION HISTORY:
**
**  09-Feb-2000  Shi-Jung Kao
**	Called _alloc_nontype instead of malloc etc...
**	Added DEMANGLE_TRACEIN/TRACEOUT.
**
**--
**/


#ifdef CXX_NT
#include "defs.h"
#include "mn_nm.h"
#include "gram_elm.h"
#include "util.h"
#else
#include "demangle.h"
#include "grammar_elements.h"
#endif

#include "demangle_internal.h"

#include <stdlib.h>
#include <stddef.h>
#include <stdio.h>
#include <string.h>

/* language element particular definitions */
#define ASCII_ZERO 48

int /* recognition status */
_unmodified_type
  (const char* s,		/* IN:  string to be parsed */
   int start,			/* IN:  index of first unparsed character */
   int* end,			/* OUT: index of last parsed character */
   struct MLD_unmodified_type** ut)	/* OUT: unmodified type pointer */
{
  /* variables particular to this language element */
  int status;
  int end_of_grammar_element;
  int temp_int;
  void* temp_ptr;
  
  DEMANGLE_TRACEIN(_unmodified_type);

  *ut = 0L;
  /* check if the we're dealing with a basic type */
  status = _basic_type(s, start, &end_of_grammar_element, &temp_int);
  switch (status)
  {
    case MLD_SOMETHING_RECOGNIZED:
      *ut = (struct MLD_unmodified_type*) 
	malloc(sizeof(struct MLD_unmodified_type));
      if (ut == 0L) return MLD_ERROR;

      (*ut)->shared_id = 0L;
      DEMANGLE_RECORD_ALLOC(*ut);
      
      if (temp_int == NUMBER_OF_BASIC_TYPES || 
	  temp_int == NUMBER_OF_BASIC_TYPES + 1) {
	/* This is a nontype, recreate a MLD_nontype instead. */
	struct MLD_nontype* nt = 0L;
	nt = _alloc_nontype();
	if (nt == 0L) return MLD_ERROR;
	nt->nontype_name = (char*) malloc(end_of_grammar_element-start+2);
	if (nt->nontype_name == 0L) {
	  _delete_nontype(nt);
	  DEMANGLE_TRACEOUT(_unmodified_type);
	  return MLD_ERROR;
	}
	DEMANGLE_RECORD_ALLOC(nt->nontype_name);
	strncpy(nt->nontype_name, &s[start], end_of_grammar_element-start+1);
	nt->nontype_name[end_of_grammar_element-start+1] = '\0';

	(*ut)->type_of_type = MLD_NONTYPE_TYPE;
	(*ut)->the_type.nontype = (struct MLD_nontype*) nt;
	
      } else {
	(*ut)->type_of_type = MLD_BASIC_TYPE;
	(*ut)->the_type.basic = temp_int;
      }
      *end = end_of_grammar_element;
      DEMANGLE_TRACEOUT(_unmodified_type);
      return MLD_SOMETHING_RECOGNIZED;
    break;
    
    case MLD_ERROR:
      *ut = 0L;
      *end = start - 1;
      DEMANGLE_TRACEOUT(_unmodified_type);
      return MLD_ERROR;
    break;
  }
  
  /* check if the we're dealing with a complex type */
  status = _parse_qualifications(s, start, &end_of_grammar_element, 
                                (struct MLD_qualification**) &temp_ptr);
  switch (status) {
  case MLD_SOMETHING_RECOGNIZED:
    *ut = (struct MLD_unmodified_type*) 
	malloc(sizeof(struct MLD_unmodified_type));
    DEMANGLE_RECORD_ALLOC(*ut);
      
    (*ut)->type_of_type = MLD_COMPLEX_TYPE;
    (*ut)->the_type.complex = (struct MLD_qualification*) temp_ptr;
    (*ut)->shared_id = 0L;

    *end = end_of_grammar_element;
    DEMANGLE_TRACEOUT(_unmodified_type);
    return MLD_SOMETHING_RECOGNIZED;
    break;
    
  case MLD_ERROR:
    *ut = 0L;
    *end = start - 1;
    DEMANGLE_TRACEOUT(_unmodified_type);
    return MLD_ERROR;
    break;

  default:
    break;
  }
  
  /* check if the we're dealing with a pointer type */
  status = _pointer_type(s, start, &end_of_grammar_element, 
                        (struct MLD_pointer**) &temp_ptr);
  switch (status)
  {
    case MLD_SOMETHING_RECOGNIZED:
      *ut = (struct MLD_unmodified_type*) 
	malloc(sizeof(struct MLD_unmodified_type));
      DEMANGLE_RECORD_ALLOC(*ut);
      
      (*ut)->type_of_type = MLD_POINTER_TYPE;
      (*ut)->the_type.pointer = (struct MLD_pointer*) temp_ptr;
      (*ut)->shared_id = 0L;

      *end = end_of_grammar_element;
      DEMANGLE_TRACEOUT(_unmodified_type);
      return MLD_SOMETHING_RECOGNIZED;
      break;
    
    case MLD_ERROR:
      *ut = 0L;
      *end = start - 1;
      DEMANGLE_TRACEOUT(_unmodified_type);
      return MLD_ERROR;
      break;

    default:
      break;
  }
  
  /* check if the we're dealing with a reference type */
  status = _reference_type(s, start, &end_of_grammar_element, 
                        (struct MLD_reference**) &temp_ptr);
  switch (status)
  {
    case MLD_SOMETHING_RECOGNIZED:
      *ut = (struct MLD_unmodified_type*) 
	malloc(sizeof(struct MLD_unmodified_type));
      DEMANGLE_RECORD_ALLOC(*ut);
      
      (*ut)->type_of_type = MLD_REFERENCE_TYPE;
      (*ut)->the_type.reference = (struct MLD_reference*) temp_ptr;
      (*ut)->shared_id = 0L;

      *end = end_of_grammar_element;
      DEMANGLE_TRACEOUT(_unmodified_type);
      return MLD_SOMETHING_RECOGNIZED;
    break;
    
    case MLD_ERROR:
      *ut = 0L;
      *end = start - 1;
      DEMANGLE_TRACEOUT(_unmodified_type);
      return MLD_ERROR;
    break;
  }
  
  /* check if the we're dealing with a array type */
  status = _array_type(s, start, &end_of_grammar_element, 
                        (struct MLD_array**) &temp_ptr);
  switch (status)
  {
    case MLD_SOMETHING_RECOGNIZED:
      *ut = (struct MLD_unmodified_type*) 
	malloc(sizeof(struct MLD_unmodified_type));
      DEMANGLE_RECORD_ALLOC(*ut);
      
      (*ut)->type_of_type = MLD_ARRAY_TYPE;
      (*ut)->the_type.array = (struct MLD_array*) temp_ptr;
      (*ut)->shared_id = 0L;

      *end = end_of_grammar_element;
      DEMANGLE_TRACEOUT(_unmodified_type);
      return MLD_SOMETHING_RECOGNIZED;
    break;
    
    case MLD_ERROR:
      *ut = 0L;
      *end = start - 1;
      DEMANGLE_TRACEOUT(_unmodified_type);
      return MLD_ERROR;
    break;
  }
  
  /* check if the we're dealing with a pointer to member type */
  status = _pointer_to_member_type(s, start, &end_of_grammar_element, 
                                  (struct MLD_pointer_to_member**) &temp_ptr);
  switch (status)
  {
    case MLD_SOMETHING_RECOGNIZED:
      *ut = (struct MLD_unmodified_type*) 
	malloc(sizeof(struct MLD_unmodified_type));
      DEMANGLE_RECORD_ALLOC(*ut);
      
      (*ut)->type_of_type = MLD_POINTER_TO_MEMBER_TYPE;
      (*ut)->the_type.ptr_to_member = (struct MLD_pointer_to_member*) temp_ptr;
      (*ut)->shared_id = 0L;

      *end = end_of_grammar_element;
      DEMANGLE_TRACEOUT(_unmodified_type);
      return MLD_SOMETHING_RECOGNIZED;
    break;
    
    case MLD_ERROR:
      *ut = 0L;
      *end = start - 1;
      DEMANGLE_TRACEOUT(_unmodified_type);
      return MLD_ERROR;
    break;
  }
  
  /* check if the we're dealing with a function type */
  status = _function_type(s, start, &end_of_grammar_element, 
                         (struct MLD_function**) &temp_ptr);
  switch (status)
  {
    case MLD_SOMETHING_RECOGNIZED:
      *ut = (struct MLD_unmodified_type*) 
	malloc(sizeof(struct MLD_unmodified_type));
      DEMANGLE_RECORD_ALLOC(*ut);
      
      (*ut)->type_of_type = MLD_FUNCTION_TYPE;
      (*ut)->the_type.function = (struct MLD_function*) temp_ptr;
      (*ut)->shared_id = 0L;

      *end = end_of_grammar_element;
      DEMANGLE_TRACEOUT(_unmodified_type);
      return MLD_SOMETHING_RECOGNIZED;
    break;
    
    case MLD_ERROR:
      *ut = 0L;
      *end = start - 1;
      DEMANGLE_TRACEOUT(_unmodified_type);
      return MLD_ERROR;
    break;
  }
  
  /* Nothing Recognized */
  *ut = 0L;
  *end = start - 1;
  DEMANGLE_TRACEOUT(_unmodified_type);
  return MLD_NOTHING_RECOGNIZED;
}
